#include "soft_drinks.h"
#include <iostream>
#include <algorithm>
#include <vector>

using namespace std;

using ll = long long;

const int MAXN = 100000;
const int MAXQ = 100000;
const ll MAXVAL = 1000000000;
const ll INF = MAXVAL+2;

struct Frac
{
	ll a, b;
	Frac(ll a, ll b) : a(a), b(b)
	{
		if (b < 0) {
			this->a = -a;
			this->b = -b;
		}
	}
	Frac() : a(0), b(1) {}
	double decimal() const
	{
		return ((double)(a)) / b;
	}
	
	ll cmp(const Frac &other) const
	{
		return a * other.b - b * other.a;
	}
	
	bool operator < (const Frac &other) const
	{
		return cmp(other) < 0;
	}
	bool operator > (const Frac &other) const
	{
		return cmp(other) > 0;
	}
	bool operator <= (const Frac &other) const
	{
		return cmp(other) <= 0;
	}
	bool operator >= (const Frac &other) const
	{
		return cmp(other) >= 0;
	}
	bool operator == (const Frac &other) const
	{
		return cmp(other) == 0;
	}
	bool operator != (const Frac &other) const
	{
		return cmp(other) != 0;
	}
};

struct Vec
{
	ll x, y;
	Vec(ll x, ll y) : x(x), y(y) {}
	Vec() : x(), y() {}
	
	Frac slope() const
	{
		return Frac(y, x);
	}
};

int n;
Vec v[MAXN+2];

int vecLowerBound(ll x)
{
	int st = 0, dr = n+1;
	while (dr-st > 1) {
		int mij = (st+dr) / 2;
		if (v[mij].x >= x) {
			dr = mij;
		}
		else {
			st = mij;
		}
	}
	return dr;
}

Frac getSumLengths(const Vec &v1, const Vec &v2, const Vec &vt)
{
	return Frac(
	vt.x * (v2.y - v1.y) + vt.y * (v1.x - v2.x),
	v1.x * v2.y - v2.x * v1.y
	);
}

template<int MAXSZ>
struct CoolAint
{
	struct Data
	{
		int vs[2];
		int st, dr;
		Data() : vs(), st(), dr() {}
	};
	Data v[MAXSZ];
	int n, root, nrnodes = 0;
	
	void pull(Data &node, const Data &l, const Data &r)
	{
		node.vs[0] = node.vs[1] = 0;
		int j = 0;
		for (int i = 0; i < 2; i++) {
			if (l.vs[i]) {
				node.vs[j++] = l.vs[i];
			}
		}
		for (int i = 0; i < 2; i++) {
			if (r.vs[i]) {
				if (j < 2) {
					node.vs[j++] = r.vs[i];
				}
				else {
					Frac res = getSumLengths(::v[node.vs[0]], ::v[r.vs[i]], ::v[node.vs[1]]);
					if (res > Frac(1, 0)) {
						node.vs[1] = r.vs[i];
					}
				}
			}
		}
	}
	
	void build(int &node, int l, int r)
	{
		node = ++nrnodes;
		if (l == r) {
			v[node].vs[0] = l;
			v[node].vs[1] = 0;
			return;
		}
		int mij = (l+r) / 2;
		build(v[node].st, l, mij);
		build(v[node].dr, mij+1, r);
		pull(v[node], v[v[node].st], v[v[node].dr]);
	}
	void init(int n)
	{
		this->n = n;
		nrnodes = 0;
		build(root, 1, n);
	}
	
	void update(int &node, int l, int r, int poz, int val)
	{
		v[++nrnodes] = v[node];
		node = nrnodes;
		if (l == r) {
			v[node].vs[0] = val;
			v[node].vs[1] = 0;
			return;
		}
		int mij = (l+r) / 2;
		if (poz <= mij) {
			update(v[node].st, l, mij, poz, val);
		}
		else {
			update(v[node].dr, mij+1, r, poz, val);
		}
		pull(v[node], v[v[node].st], v[v[node].dr]);
	}
	Data query(int node, int l, int r, int ql, int qr)
	{
		if (ql <= l && r <= qr) {
			return v[node];
		}
		int mij = (l+r) / 2;
		if (mij+1 > qr) return query(v[node].st, l, mij, ql, qr);
		if (mij < ql) return query(v[node].dr, mij+1, r, ql, qr);
		Data aux;
		pull(aux, query(v[node].st, l, mij, ql, qr), query(v[node].dr, mij+1, r, ql, qr));
		return aux;
	}
};
CoolAint<7000000> aint;

int roots[MAXN+2];

void sortAndStuff()
{
	sort(v+1, v+n+1, [&](const Vec &a, const Vec &b)
	{
		if (a.x != b.x) return a.x < b.x;
		return a.y < b.y;
	});
	int m = 1;
	for (int i = 2; i <= n; i++) {
		if (v[i].x != v[i-1].x) {
			v[++m] = v[i];
		}
	}
	n = m;
}

void hullStuff()
{
	vector<int> stk;
	for (int i = n; i >= 1; i--) {
		while (!stk.empty() && v[i].y <= v[stk.back()].y) {
			aint.update(aint.root, 1, n, stk.back(), 0);
			stk.pop_back();
		}
		stk.push_back(i);
		roots[i] = aint.root;
	}
}

void init(const vector<int> &A, const vector<int> &B)
{
	n = A.size();
	for (int i = 1; i <= n; i++) {
		v[i] = Vec(A[i-1], B[i-1]);
	}
	//cerr << "read all!\n";
	sortAndStuff();
	//cerr << "sorted!\n";
	aint.init(n);
	//cerr << "aint is good!\n";
	hullStuff();
	//cerr << "and hull too. yeah.\n";
}

double friendDrink(int maxx, int maxy, int lx, int rx)
{
	//cerr << "oh a query!\n";
	Vec vt = Vec(maxx, maxy);
	int l = vecLowerBound(lx);
	int r = vecLowerBound(rx) - 1;
	//cerr << l << ' ' << r << '\n';
	if (l > r) {
		return 0;
	}
	auto res = aint.query(roots[l], 1, n, l, r);
	//cerr << "wow i could actually query, cool\n";
	if (!res.vs[1] || v[res.vs[0]].slope() <= vt.slope()) {
		Frac xfrac = Frac(vt.x, v[res.vs[0]].x);
		Frac yfrac = Frac(vt.y, v[res.vs[0]].y);
		if (xfrac < yfrac) {
			return ((double)(vt.x)) / v[res.vs[0]].x;
		}
		return ((double)(vt.y)) / v[res.vs[0]].y;
	}
	if (v[res.vs[1]].slope() >= vt.slope()) {
		Frac xfrac = Frac(vt.x, v[res.vs[1]].x);
		Frac yfrac = Frac(vt.y, v[res.vs[1]].y);
		if (xfrac < yfrac) {
			return ((double)(vt.x)) / v[res.vs[1]].x;
		}
		return ((double)(vt.y)) / v[res.vs[1]].y;
	}
	/// altfel... cred ca putem face chestia
	//cerr << "guh its complicated\n";
	Frac ans = getSumLengths(v[res.vs[0]], v[res.vs[1]], vt);
	return ans.decimal();
}
