#include "soft_drinks.h"
#include <iostream>
#include <algorithm>
#include <vector>
#include <deque>

using namespace std;

using ll = long long;

const int MAXN = 100000;
const int MAXQ = 100000;

struct Frac
{
	ll a, b;
	Frac(ll a, ll b) : a(a), b(b)
	{
		if (b < 0) {
			this->a = -a;
			this->b = -b;
		}
	}
	Frac() : a(0), b(1) {}
	double decimal() const
	{
		return ((double)(a)) / b;
	}
	
	ll cmp(const Frac &other) const
	{
		return a * other.b - b * other.a;
	}
	
	bool operator < (const Frac &other) const
	{
		return cmp(other) < 0;
	}
	bool operator > (const Frac &other) const
	{
		return cmp(other) > 0;
	}
	bool operator <= (const Frac &other) const
	{
		return cmp(other) <= 0;
	}
	bool operator >= (const Frac &other) const
	{
		return cmp(other) >= 0;
	}
	bool operator == (const Frac &other) const
	{
		return cmp(other) == 0;
	}
	bool operator != (const Frac &other) const
	{
		return cmp(other) != 0;
	}
};

struct Vec
{
	ll x, y;
	Vec(ll x, ll y) : x(x), y(y) {}
	Vec() : x(), y() {}
	
	ll dotProduct(const Vec &other) const
	{
		return x * other.x + y * other.y;
	}
	ll crossProduct(const Vec &other) const
	{
		return x * other.y - y * other.x;
	}
	Frac slope() const
	{
		return Frac(y, x);
	}
	
	Vec operator - (const Vec &other) const
	{
		return Vec(x - other.x, y - other.y);
	}
	Vec operator + (const Vec &other) const
	{
		return Vec(x + other.x, y + other.y);
	}
};

int n;
Vec v[MAXN+2];

int vecLowerBound(ll x)
{
	int st = 0, dr = n+1;
	while (dr-st > 1) {
		int mij = (st+dr) / 2;
		if (v[mij].x >= x) {
			dr = mij;
		}
		else {
			st = mij;
		}
	}
	return dr;
}

Frac getSumLengths(const Vec &v1, const Vec &v2, const Vec &vt)
{
	return Frac(
	vt.x * (v2.y - v1.y) + vt.y * (v1.x - v2.x),
	v1.x * v2.y - v2.x * v1.y
	);
}
Frac getTouchingLen(const Vec &v, const Vec &vt)
{
	Frac xfrac = Frac(vt.x, v.x);
	Frac yfrac = Frac(vt.y, v.y);
	if (!v.x) return yfrac;
	if (!v.y) return xfrac;
	return (xfrac < yfrac) ? xfrac : yfrac;
}

struct LowerHull
{
	deque<int> hull;
	void clear()
	{
		hull.clear();
	}
	void pushBack(int poz)
	{
		if (hull.empty()) {
			hull.push_back(poz);
			return;
		}
		if (v[poz].y >= v[hull.back()].y) return;
		while (hull.size() >= 2
		&& (v[hull.back()] - v[hull[hull.size()-2]]).crossProduct(v[poz] - v[hull.back()]) <= 0) {
			hull.pop_back();
		}
		hull.push_back(poz);
	}
	void pushFront(int poz)
	{
		if (hull.empty()) {
			hull.push_front(poz);
			return;
		}
		if (v[poz].slope() <= v[hull.front()].slope()) return;
		while (hull.size() >= 2
		&& (v[hull[0]] - v[hull[1]]).crossProduct(v[poz] - v[hull[0]]) >= 0) {
			hull.pop_front();
		}
		hull.push_front(poz);
	}
	Frac getAns(const Vec &vt)
	{
		if (hull.empty()) return Frac(0, 1);
		if (v[hull.front()].slope() <= vt.slope()) {
			return getTouchingLen(v[hull.front()], vt);
		}
		if (v[hull.back()].slope() >= vt.slope()) {
			return getTouchingLen(v[hull.back()], vt);
		}
		int st = 0, dr = hull.size();
		while (dr-st > 1) {
			int mij = (st+dr) / 2;
			if (v[hull[mij]].slope() >= vt.slope()) {
				st = mij;
			}
			else {
				dr = mij;
			}
		}
		return getSumLengths(v[hull[st]], v[hull[st+1]], vt);
	}
};
LowerHull allHull, auxHull;

template<int MAXSZ>
struct CoolAint
{
	struct Data
	{
		int vs[2];
		int st, dr;
		Data() : vs(), st(), dr() {}
	};
	Data v[MAXSZ];
	int n, root, nrnodes = 0;
	
	void pull(Data &node, const Data &l, const Data &r)
	{
		node.vs[0] = node.vs[1] = 0;
		int j = 0;
		for (int i = 0; i < 2; i++) {
			if (l.vs[i]) {
				node.vs[j++] = l.vs[i];
			}
		}
		for (int i = 0; i < 2; i++) {
			if (r.vs[i]) {
				if (j < 2) {
					node.vs[j++] = r.vs[i];
				}
				else {
					Frac res = getSumLengths(::v[node.vs[0]], ::v[r.vs[i]], ::v[node.vs[1]]);
					if (res > Frac(1, 1)) {
						node.vs[1] = r.vs[i];
					}
				}
			}
		}
	}
	
	void build(int &node, int l, int r)
	{
		node = ++nrnodes;
		if (l == r) {
			v[node].vs[0] = l;
			v[node].vs[1] = 0;
			return;
		}
		int mij = (l+r) / 2;
		build(v[node].st, l, mij);
		build(v[node].dr, mij+1, r);
		pull(v[node], v[v[node].st], v[v[node].dr]);
	}
	void init(int n)
	{
		this->n = n;
		nrnodes = 0;
		build(root, 1, n);
	}
	
	void update(int &node, int l, int r, int poz, int val)
	{
		v[++nrnodes] = v[node];
		node = nrnodes;
		if (l == r) {
			v[node].vs[0] = val;
			v[node].vs[1] = 0;
			return;
		}
		int mij = (l+r) / 2;
		if (poz <= mij) {
			update(v[node].st, l, mij, poz, val);
		}
		else {
			update(v[node].dr, mij+1, r, poz, val);
		}
		pull(v[node], v[v[node].st], v[v[node].dr]);
	}
	Data query(int node, int l, int r, int ql, int qr)
	{
		if (ql <= l && r <= qr) {
			return v[node];
		}
		int mij = (l+r) / 2;
		if (mij+1 > qr) return query(v[node].st, l, mij, ql, qr);
		if (mij < ql) return query(v[node].dr, mij+1, r, ql, qr);
		Data aux;
		pull(aux, query(v[node].st, l, mij, ql, qr), query(v[node].dr, mij+1, r, ql, qr));
		return aux;
	}
};
//CoolAint<7000000> aint;

//int roots[MAXN+2];

int bestDiag;

void sortAndStuff()
{
	sort(v+1, v+n+1, [&](const Vec &a, const Vec &b)
	{
		if (a.x != b.x) return a.x < b.x;
		return a.y < b.y;
	});
	int m = 1;
	for (int i = 2; i <= n; i++) {
		if (v[i].x != v[i-1].x) {
			v[++m] = v[i];
		}
	}
	n = m;
}

void hullStuff()
{
	for (int i = 1; i <= n; i++) {
		allHull.pushBack(i);
	}
}

void init(const vector<signed> &A, const vector<signed> &B)
{
	n = A.size();
	for (int i = 1; i <= n; i++) {
		v[i] = Vec(A[i-1], B[i-1]);
	}
	//cerr << "read all!\n";
	sortAndStuff();
	//cerr << "sorted!\n";
	hullStuff();
	//cerr << "and hull too. yeah.\n";
	bestDiag = 1;
	for (int i = 1; i <= n; i++) {
		if (max(v[i].x, v[i].y) < max(v[bestDiag].x, v[bestDiag].y)) {
			bestDiag = i;
		}
	}
}

double friendDrink(signed maxx, signed maxy, signed lx, signed rx)
{
	//cerr << "oh a query!\n";
	if (lx > rx) return 0;
	Vec vt = Vec(maxx, maxy);
	int l = vecLowerBound(lx);
	int r = vecLowerBound(rx+1) - 1;
	//cerr << l << ' ' << r << '\n';
	if (l > r) {
		return 0;
	}
	if (l == r) {
		return getTouchingLen(v[l], vt).decimal();
	}
	if (l == 1 && r == n) {
		return allHull.getAns(vt).decimal();
	}
	if (vt.x == 0) {
		if (v[l].x) return 0;
		return getTouchingLen(v[l], vt).decimal();
	}
	//cerr << "oh cool aux hull\n";
	auxHull.clear();
	for (int i = l; i <= r; i++) {
		auxHull.pushBack(i);
	}
	return auxHull.getAns(vt).decimal();
}
