#include <iostream>
#include <algorithm>
#include <vector>
#include <deque>
#include <random>
#include <chrono>

using namespace std;

using ll = long long;

mt19937 rng(chrono::steady_clock::now().time_since_epoch().count());

struct Edge
{
	int x, y, w;
	int other(int node) const
	{
		return (node == x) ? y : x;
	}
};

const int MAXN = 200000;

int n, k, root, p[MAXN+2], pedge[MAXN+2], depth[MAXN+2], wd[MAXN+2], jump[MAXN+2];
Edge edges[MAXN+2];
vector<int> nb[MAXN+2], dp[MAXN+2];
vector<int>::iterator it;

int getLca(int a, int b)
{
	if (depth[a] < depth[b]) swap(a, b);
	while (depth[a] > depth[b]) {
		if (depth[jump[a]] >= depth[b]) {
			a = jump[a];
		}
		else {
			a = p[a];
		}
	}
	while (a != b) {
		if (jump[a] != jump[b]) {
			a = jump[a], b = jump[b];
		}
		else {
			a = p[a], b = p[b];
		}
	}
	return a;
}

void dpDfs(int node)
{
	dp[node].push_back(1);
	for (auto x : nb[node]) {
		if (!p[x]) {
			p[x] = node;
			depth[x] = depth[node] + 1;
			dpDfs(x);
			dp[x].push_back(dp[x].back());
			if (dp[x].size() > dp[node].size()) {
				dp[node].swap(dp[x]);
			}
			for (int i = 0; i < dp[x].size(); i++) {
				dp[node][i + dp[node].size() - dp[x].size()] += dp[x][i];
			}
			dp[x].clear();
		}
	}
}

void dfs(int node)
{
	bool bigjump = (depth[node] - depth[jump[node]] == depth[jump[node]] - depth[jump[jump[node]]]);
	for (auto eid : nb[node]) {
		int x = edges[eid].other(node);
		if (!p[x]) {
			p[x] = root;
			pedge[x] = eid;
			depth[x] = depth[node] + 1;
			wd[x] = wd[node] + edges[eid].w;
			jump[x] = bigjump ? jump[jump[node]] : node;
			dfs(x);
		}
	}
}

void dfsHelper(int node)
{
	fill(p+1, p+n+1, 0);
	p[node] = node;
	pedge[node] = 0;
	depth[node] = wd[node] = 0;
	jump[node] = node;
	dfs(node);
}

int main()
{
	cin.tie(0)->sync_with_stdio(0);
	cin >> n >> k;
	for (int i = 1; i < n; i++) {
		cin >> edges[i].x >> edges[i].y >> edges[i].w;
		nb[edges[i].x].push_back(i);
		nb[edges[i].y].push_back(i);
	}
	if (n == 1) {
		cout << 0;
		return 0;
	}
	root = 1;
	dfsHelper(root);
	for (int i = 2; i <= n; i++) {
		if (wd[i] > wd[root]) {
			root = i;
		}
	}
	dfsHelper(root);
	int maxi = 0, lca = root;
	for (int i = 1; i <= n; i++) {
		if (wd[i] > maxi) {
			maxi = wd[i];
			lca = i;
		}
		else if (wd[i] == maxi) {
			lca = getLca(lca, i);
		}
	}
	if (!k) {
		cout << maxi;
		return 0;
	}
	if (lca == root) {
		cout << maxi;
		return 0;
	}
	edges[pedge[lca]].w--;
	root = 1;
	dfsHelper(root);
	for (int i = 2; i <= n; i++) {
		if (wd[i] > wd[root]) {
			root = i;
		}
	}
	dfsHelper(root);
	maxi = 0;
	for (int i = 1; i <= n; i++) {
		maxi = max(maxi, wd[i]);
	}
	cout << maxi;
	return 0;
}
