#include <iostream>
#include <algorithm>
#include <vector>
#include <deque>
#include <random>
#include <chrono>

using namespace std;

using ll = long long;

mt19937 rng(chrono::steady_clock::now().time_since_epoch().count());

struct Edge
{
	int x, y, w;
	int other(int node) const
	{
		return (node == x) ? y : x;
	}
};

const int MAXN = 200000;

int n, k, root, p[MAXN+2], pedge[MAXN+2], depth[MAXN+2], wd[MAXN+2], jump[MAXN+2];
Edge edges[MAXN+2];
int lazy[MAXN+2];
vector<int> nb[MAXN+2], ch[MAXN+2];
deque<int> dp[MAXN+2];

int getLca(int a, int b)
{
	if (depth[a] < depth[b]) swap(a, b);
	while (depth[a] > depth[b]) {
		if (depth[jump[a]] >= depth[b]) {
			a = jump[a];
		}
		else {
			a = p[a];
		}
	}
	while (a != b) {
		if (jump[a] != jump[b]) {
			a = jump[a], b = jump[b];
		}
		else {
			a = p[a], b = p[b];
		}
	}
	return a;
}

int dpDfs(int node, int diam)
{
	dp[node].push_back(1);
	if (ch[node].empty()) return 0;
	int ans = 0;
	for (auto x : ch[node]) {
		ans += dpDfs(x, diam);
		dp[x].push_back(dp[x].back());
	}
	sort(ch[node].begin(), ch[node].end(), [&](int a, int b) { return dp[a].size() > dp[b].size(); });
	if (ch[node].size() == 1) {
		int x = ch[node][0];
		dp[node].swap(dp[x]);
		swap(lazy[node], lazy[x]);
		dp[x].clear();
		while (dp[node].size() - 1 > diam) {
			ans += dp[node].front() - lazy[node];
			lazy[node] = dp[node].front();
			dp[node].pop_front();
		}
		return ans;
	}
	while (dp[ch[node][0]].size() + dp[ch[node][1]].size() - 2 > diam) {
		int sz = dp[ch[node][0]].size();
		for (int i = (dp[ch[node][0]].size() == dp[ch[node][1]].size()); i < ch[node].size(); i++) {
			int x = ch[node][i];
			if (dp[x].size() != sz) break;
			ans += dp[x].front() - lazy[x];
			lazy[x] = dp[x].front();
			dp[x].pop_front();
		}
	}
	for (auto x : ch[node]) {
		dp[x].push_back(dp[x].empty() ? 0 : dp[x].back());
		if (dp[x].size() > dp[node].size()) {
			dp[node].swap(dp[x]);
			swap(lazy[node], lazy[x]);
		}
		for (int i = 0; i < dp[x].size(); i++) {
			dp[node][i + dp[node].size() - dp[x].size()] += dp[x][i] - lazy[x];
		}
		dp[x].clear();
		lazy[x] = 0;
	}
	return ans;
}

int dpDfsHelper(int node, int diam)
{
	for (int i = 1; i <= n; i++) {
		dp[i].clear();
		lazy[i] = 0;
	}
	return dpDfs(node, diam);
}

void dfs(int node)
{
	bool bigjump = (depth[node] - depth[jump[node]] == depth[jump[node]] - depth[jump[jump[node]]]);
	for (auto eid : nb[node]) {
		int x = edges[eid].other(node);
		if (!p[x]) {
			p[x] = node;
			ch[node].push_back(x);
			pedge[x] = eid;
			depth[x] = depth[node] + 1;
			wd[x] = wd[node] + edges[eid].w;
			jump[x] = bigjump ? jump[jump[node]] : node;
			dfs(x);
		}
	}
}

void dfsHelper(int node)
{
	fill(p+1, p+n+1, 0);
	for (int i = 1; i <= n; i++) {
		ch[i].clear();
	}
	p[node] = node;
	pedge[node] = 0;
	depth[node] = wd[node] = 0;
	jump[node] = node;
	dfs(node);
}

int main()
{
	cin.tie(0)->sync_with_stdio(0);
	cin >> n >> k;
	for (int i = 1; i < n; i++) {
		cin >> edges[i].x >> edges[i].y >> edges[i].w;
		nb[edges[i].x].push_back(i);
		nb[edges[i].y].push_back(i);
	}
	if (k >= n-1) {
		cout << 0;
		return 0;
	}
	root = 1;
	dfsHelper(root);
	for (int i = 1; i <= n; i++) {
		if (wd[i] > wd[root]) {
			root = i;
		}
	}
	dfsHelper(root);
	int st = 0, dr = n;
	while (dr-st > 1) {
		int mij = (st+dr) / 2;
		int res = dpDfsHelper(root, mij);
		if (res <= k) {
			st = mij;
		}
		else {
			dr = mij;
		}
	}
	cout << st;
	return 0;
}
