#include <iostream>
#include <set>
#include <vector>
#include <cassert>
#include <algorithm>
#include <random>
#include <chrono>
#include <functional>
#include <ranges>
#define all(x) (x).begin(),(x).end()

using namespace std;

using ll = long long;
// #define int ll
#define sz(x) ((int)(x).size())

using pii = pair<int, int>;
using tii = tuple<int, int, int>;

const int inf = 1e9 + 5;


template<typename T>
struct AINT {
   vector<T> aint;
   int n;
   void init(int n_) {
      n = n_;
      aint.assign(2 * n + 5, T());
   }
   template<class CB> void walk(CB&& cb) { walk(cb, 1, n); }
   template<class CB> void walk(CB&& cb, int l, int r) { walk(cb, l, r, 1, 1, n); }
   template<class CB> void walk(CB&& cb, int l, int r, int node, int cl, int cr) { 
      if(cr < l || r < cl) return;
      if(l <= cl && cr <= r && !cb(aint[node], cl, cr)) return;
      int mid = (cl + cr) >> 1, L = node + 1, R = node + (mid - cl + 1) * 2;
      aint[node].push(aint[L], aint[R]);
      walk(cb, l, r, L, cl, mid);
      walk(cb, l, r, R, mid + 1, cr);
      aint[node].pull(aint[L], aint[R]);
   }
};

struct MaxAdd {
   int mx;
   int lazy;
   MaxAdd(): mx(-inf), lazy(0) {;}
   MaxAdd(int a): mx(a), lazy(0) {;}
   void apply(int x) {
      mx += x;
      lazy += x;
   }
   void pull(const MaxAdd& L, const MaxAdd& R) {
      mx = max(L.mx, R.mx);
   }
   void push(MaxAdd& L, MaxAdd& R) {
      if(!lazy) return;
      L.apply(lazy);
      R.apply(lazy);
      lazy = 0;
   }
};

namespace G {
   vector<vector<int>> g;
   vector<int> d, pin, pout, p, preord;
   int inp;
   
   void init(int n) { g.assign(n, vector<int>{}); }
   int newnode() { g.emplace_back(); return sz(g) - 1; }
   void addedge(int a, int b) { g[b].emplace_back(a), g[a].emplace_back(b); }
      
   void startgraph() {
      d.resize(sz(g));
      pin.resize(sz(g));
      p.resize(sz(g));
      pout.resize(sz(g));
   }
   void initd(int node, int f) {
      d[node] = d[f] + 1;
      p[node] = f;
      for(auto x : g[node]) {
	 if(x == f) continue;
	 initd(x, node);
      }
      return;
   }
   // #warning mi-e lene aici:
   int findcentr() {
      int x = 0, y = 1;
      for(int step = 0; step < 2; step++) {
	 fill(all(d), 0);
	 initd(x, x);
	 // for(auto x : d) cerr << x << ' ';
	 // cerr << '\n';
	 for(int i = 0; i < sz(g); i++)
	    if(d[i] > d[y]) y = i;
	 swap(x, y);
      }
      fill(all(d), 0);
      initd(x, x);
      int D = d[y];
      D /= 2;
      // cerr << x << ' ' << y << '\n';
      for(int i = 0; i < D; i++, y = p[y]);
      return y;
   }
   
   void initall(int node, int f) {
      d[node] = d[f] + 1;
      pin[node] = inp++;
      preord.emplace_back(node);
      for(auto x : g[node]) {
	 if(x == f) continue;
	 initall(x, node);
      }
      pout[node] = inp - 1;
      return;
   }
   void initall(int root) {
      inp = 1;
      fill(all(d), 0);
      preord.clear();
      preord.emplace_back(-1);
      d[root] = -1;
      initall(root, root);
   }
   bool isanc(int f, int node) { return pin[f] <= pin[node] && pout[node] <= pout[f]; }
}

const int nmax = 1e6 + 5;
vector<int> dial[nmax];

struct DistsMaintainer : AINT<MaxAdd> {
   void addsub(int node, int val) {
      walk([&](auto &a, int cl, int cr) {
	 a.apply(val);
	 return 0;
      }, G::pin[node], G::pout[node]);
   } 
   int query(int node) {
      int mx = 0;
      walk([&](auto &a, int cl, int cr) {
	 mx = max(mx, a.mx);
	 return 0;
      }, G::pin[node], G::pout[node]);
      return mx;
   }
};

signed main() {
   cin.tie(0) -> sync_with_stdio(0);
   
   int n, K;
   cin >> n >> K;
   G::init(n);
   for(int i = 1, x, y, w; i < n; i++) {
      cin >> x >> y >> w;
      --x;
      --y;
      int lst = x;
      for(int j = 1; j < w; j++) {
	 int z = G::newnode();
	 G::addedge(lst, z);
	 lst = z;
      }
      G::addedge(lst, y);
   }
   using namespace G;
   startgraph();
   n = sz(g);
   if(K >= n - 1) {
      cout << "0\n";
      return 0;
   }
   // cerr << sz(g) << '\n';
   // for(int i = 0; i < sz(g); i++) {
      // for(auto j :  g[i]) 
	 // if(i < j) cout << i << ' ' << j << '\n';
   // }
   
   int centr = findcentr();
   initall(centr);
   // cerr << centr << '\n';
   
   
   DistsMaintainer downdists;
   downdists.init(n);
   downdists.walk([&](auto &a, int cl, int cr) {
      if(cl != cr) return 1;
      a.mx = d[preord[cl]];
      return 0;
   });
   
   // for(int i = 0; i < n; i++) cerr << d[i] << ' ';
   // cerr << '\n';
   // cerr << pin[12] << ' ' << pout[12] << '\n';
   // cerr << pin[9] << ' ' << pout[9] << '\n';
   // cerr << pin[8] << ' ' << pout[8] << '\n';
   
   for(auto y : g[centr])
      dial[downdists.query(y)].emplace_back(y);
   
   for(int i = n; K && i > 0; i--) {
      while(K && sz(dial[i])) {
	 int x = dial[i].back();
	 dial[i].pop_back();
	 // cerr << "-- " << x << '\n';
	 
	 downdists.addsub(x, -1);
	 if(sz(g[x]) == 1) dial[0].emplace_back(x);
	 else {
	    for(auto y : g[x]) {
	       if(isanc(x, y))
		  dial[downdists.query(y)].emplace_back(y);
	    }
	 }
	 K--;
      }
   }
   
   for(int i = n; i >= 0; i--) {
      if(sz(dial[i])) {
	 if(sz(dial[i]) >= 2) cout << 2 * i << '\n';
	 else assert(sz(dial[i - 1])), cout << 2 * i - 1 << '\n';
	 return 0;
      }
   }
   assert(false);
   return 0;
}
