#include <iostream>
#include <set>
#include <vector>
#include <cassert>
#include <algorithm>
#include <random>
#include <chrono>
#include <functional>
#include <ranges>
#define all(x) (x).begin(),(x).end()

using namespace std;

using ll = long long;
// #define int ll
#define sz(x) ((int)(x).size())

using pii = pair<int, int>;
using tii = tuple<int, int, int>;

const int inf = 1e9 + 5;


template<typename T>
struct AINT {
   vector<T> aint;
   int n;
   void init(int n_) {
      n = n_;
      aint.assign(2 * n + 5, T());
   }
   template<class CB> void walk(CB&& cb) { walk(cb, 1, n); }
   template<class CB> void walk(CB&& cb, int l, int r) { walk(cb, l, r, 1, 1, n); }
   template<class CB> void walk(CB&& cb, int l, int r, int node, int cl, int cr) { 
      if(cr < l || r < cl) return;
      if(l <= cl && cr <= r && !cb(aint[node], cl, cr)) return;
      int mid = (cl + cr) >> 1, L = node + 1, R = node + (mid - cl + 1) * 2;
      aint[node].push(aint[L], aint[R]);
      walk(cb, l, r, L, cl, mid);
      walk(cb, l, r, R, mid + 1, cr);
      aint[node].pull(aint[L], aint[R]);
   }
};

struct MaxAdd {
   int mx;
   int lazy;
   MaxAdd(): mx(-inf), lazy(0) {;}
   MaxAdd(int a): mx(a), lazy(0) {;}
   void apply(int x) {
      mx += x;
      lazy += x;
   }
   void pull(const MaxAdd& L, const MaxAdd& R) {
      mx = max(L.mx, R.mx);
   }
   void push(MaxAdd& L, MaxAdd& R) {
      if(!lazy) return;
      L.apply(lazy);
      R.apply(lazy);
      lazy = 0;
   }
};

namespace G {
   vector<vector<pii>> g;
   vector<int> d, pin, pout, p, preord;
   int inp;
   
   void init(int n) { g.assign(n, vector<pii>{}); }
   int newnode() { g.emplace_back(); return sz(g) - 1; }
   void addedge(int a, int b, int w) { g[b].emplace_back(a, w), g[a].emplace_back(b, w); }
      
   void startgraph() {
      d.resize(sz(g));
      pin.resize(sz(g));
      p.resize(sz(g));
      pout.resize(sz(g));
   }
   void initd(int node, int f) {
      p[node] = f;
      for(auto [x, c] : g[node]) {
	 if(x == f) continue;
	 d[x] = d[node] + c;
	 initd(x, node);
      }
      return;
   }
   
   pii findcentr() {
      int x = 0, y = 1;
      for(int step = 0; step < 2; step++) {
	 fill(all(d), 0);
	 initd(x, x);
	 for(int i = 0; i < sz(g); i++)
	    if(d[i] > d[y]) y = i;
	 swap(x, y);
      }
      fill(all(d), 0);
      initd(x, x);
      int D = d[y];
      int z = y;
      // cerr << x << ' ' << y << '\t' << sz(g) << '\n';;
      for(int i = 0; (d[y] - d[p[z]]) * 2 < D; i++, z = p[z]);
      return make_pair(z, p[z]);
   }
   
   void initall(int node, int f) {
      pin[node] = inp++;
      preord.emplace_back(node);
      for(auto [x, c] : g[node]) {
	 if(x == f) continue;
	 d[x] = d[node] + c;
	 initall(x, node);
      }
      pout[node] = inp - 1;
      return;
   }
   void initall(int root) {
      inp = 1;
      fill(all(d), 0);
      preord.clear();
      preord.emplace_back(-1);
      initall(root, root);
   }
   bool isanc(int f, int node) { return pin[f] <= pin[node] && pout[node] <= pout[f]; }
}

const int nmax = 1e6 + 5;
vector<int> dial[nmax];

struct DistsMaintainer : AINT<MaxAdd> {
   void addsub(int node, int val) {
      walk([&](auto &a, int cl, int cr) {
	 a.apply(val);
	 return 0;
      }, G::pin[node], G::pout[node]);
   } 
   int query(int node) {
      int mx = 0;
      walk([&](auto &a, int cl, int cr) {
	 mx = max(mx, a.mx);
	 return 0;
      }, G::pin[node], G::pout[node]);
      return mx;
   }
};

struct Link {
   int weight;
   int parentexhaust, node;
};

signed main() {
   cin.tie(0) -> sync_with_stdio(0);
   
   int n, K;
   cin >> n >> K;
   G::init(n);
   for(int i = 1, x, y, w; i < n; i++) {
      cin >> x >> y >> w;
      --x;
      --y;
      G::addedge(x, y, w);
   }
   using namespace G;
   startgraph();
   n = sz(g);
   
   
   auto [centrA, centrB] = findcentr();
   // cerr << centrA << ' ' << centrB << '\n';
   initall(centrA);
   vector<int> atrs;
   for(auto [y, c] : g[centrA]) {
      int mx = 0;
      for(int j = pin[y]; j <= pout[y]; j++) mx = max(mx, d[preord[j]]);
      atrs.emplace_back(mx);
   }
   sort(all(atrs));
   atrs.back() -= K;
   sort(all(atrs));
   cout << rbegin(atrs)[0] + rbegin(atrs)[1] << '\n';
   
   
   // DistsMaintainer downdists;
   // downdists.init(n);
   // downdists.walk([&](auto &a, int cl, int cr) {
      // if(cl != cr) return 1;
      // a.mx = d[preord[cl]];
      // return 0;
   // });
   
   // for(auto y : g[centr])
      // dial[downdists.query(y)].emplace_back(y);
   
   // for(int i = n; K && i > 0; i--) {
      // while(K && sz(dial[i])) {
	 // int x = dial[i].back();
	 // dial[i].pop_back();
	 
	 // downdists.addsub(x, -1);
	 // if(sz(g[x]) == 1) dial[0].emplace_back(x);
	 // else {
	    // for(auto y : g[x]) {
	       // if(isanc(x, y))
		  // dial[downdists.query(y)].emplace_back(y);
	    // }
	 // }
	 // K--;
      // }
   // }
   
   // for(int i = n; i >= 0; i--) {
      // if(sz(dial[i])) {
	 // if(sz(dial[i]) >= 2) cout << 2 * i << '\n';
	 // else assert(sz(dial[i - 1])), cout << 2 * i - 1 << '\n';
	 // return 0;
      // }
   // }
   // assert(false);
   return 0;
}
