#include <vector>
#include <queue>
#include <iostream>

using namespace std;

struct edge {
    int u, v, c;
    int other( int w ) {
        return u ^ v ^ w;
    }
};

struct diameter {
    int len, u, v, l;
};

const int MAX_N = 2e5;
int maxDepth[MAX_N + 1], maxNode[MAX_N + 1];
int depth[MAX_N + 1];
int parent[MAX_N + 1], parentEdge[MAX_N + 1];
edge edges[MAX_N];
vector<int> adj[MAX_N + 1];

diameter diam;
void findDiam( int u, int p ) {
    int max1 = 0, max2 = 0, nod1 = u, nod2 = -1;
    for ( int e: adj[u] ) {
        int v = edges[e].other( u );
        if ( v == p )
            continue;
        int c = edges[e].c;
        depth[v] = depth[u] + c;
        parentEdge[v] = e;
        parent[v] = u;
        findDiam( v, u );
        if ( maxDepth[v] + c > max1 ) {
            max2 = max1;
            nod2 = nod1;
            max1 = maxDepth[v] + c;
            nod1 = maxNode[v];
        } else if ( maxDepth[v] + c > max2 ) {
            max2 = maxDepth[v] + c;
            nod2 = maxNode[v];
        }
    }
    maxDepth[u] = max1;
    maxNode[u] = nod1;

    if ( max1 > diam.len ) 
        diam = { max1, nod1, u, u };
    if ( nod2 != -1 && max1 + max2 > diam.len ) 
        diam = { max1 + max2, nod1, nod2, u };
}

int findCenter( diameter d ) {
    int u = d.u, v = d.v, l = d.l;
    if ( depth[u] - depth[l] < depth[v] - depth[l] )
        swap( u, v );
    int a = d.len, b = 0, minn = d.len;
    int centre = u;
    while ( u != l ) {
        int e = parentEdge[u];
        int c = edges[e].c;
        a -= c;
        b += c;
        u = parent[u];
        if ( abs( a - b ) < minn ) {
            minn = abs( a - b );
            centre = u;
        }
    }
    return centre;
}

void dfs( int u, int p ) {
    maxDepth[u] = 0;
    for ( int e: adj[u] ) {
        int v = edges[e].other( u ), c = edges[e].c;
        if ( v == p )
            continue;
        parent[u] = p;
        parentEdge[u] = e;
        dfs( v, u );
        maxDepth[u] = max( maxDepth[u], maxDepth[v] + c );
    }
}

struct elempq {
    int maxDepth, reps, u;

    bool operator < ( const elempq &x ) const {
        return maxDepth < x.maxDepth;
    }
};
priority_queue<elempq> pq;

int main() {
    int n, k;

    cin >> n >> k;
    for ( int i = 0; i < n - 1; i++ ) {
        cin >> edges[i].u >> edges[i].v >> edges[i].c;
        adj[edges[i].u].push_back( i );
        adj[edges[i].v].push_back( i );
    }

    diam = { 0, 0, 0, 0 };
    findDiam( 1, 0 );
    int c = findCenter( diam );

    dfs( c, 0 );

    k++;
    pq.push( { maxDepth[c], 1, c } );
    //printf( "CENTRU %d\n", c );
    for ( int i = 0; i < k && !pq.empty(); i++ ) {
        elempq p = pq.top();
        pq.pop();

        p.maxDepth--;
        p.reps--;
        if ( p.u != c )
            edges[parentEdge[p.u]].c--;

        //printf( "SCAD %d %d %d\n", p.u, p.maxDepth, p.reps );

        if ( p.reps > 0 )
            pq.push( p );
        else {
            for ( int e: adj[p.u] ) {
                int v = edges[e].other( p.u );
                if ( v == parent[p.u] )
                    continue;
                pq.push( { maxDepth[v] + edges[e].c, edges[e].c, v } );
            }
        }
    }

    diam = { 0, 0, 0, 0 };
    findDiam( 1, 0 );

    cout << diam.len << "\n";
}
