#include <bits/stdc++.h>

using namespace std;

const int NMAX = 1e5;

int a[NMAX];
int b[NMAX];
int n;


vector <int> hull;
vector <int> candSt[NMAX + 1];
vector <int> candDr[NMAX + 1];

int minB = 1e9;

double minDrink( int i, int Ma, int Mb ) {
    if ( b[i] == 0 ) return (double)Ma / a[i];
    if ( a[i] == 0 ) return (double)Mb / b[i];
    return min( (double)Ma / a[i], (double)Mb / b[i] );
}
bool isBetter( int i, int j ) {
    return a[i] <= a[j] && b[i] <= b[j];
}
double min2Drink( int i, int j, double Ma, double Mb ) {
    if ( i == j ) return minDrink( i, Ma, Mb );
    if ( isBetter( i, j ) || isBetter( j, i ) ) return 0;
    double t = (double)( Mb * a[j] - Ma * b[j] ) / ( Ma * ( b[i] - b[j] ) - Mb * ( a[i] - a[j] ) );

    if ( 0 <= t && t <= 1 ) {
        //assert( abs( (double)Ma / ( a[j] + t * ( a[i] - a[j] ) ) - (double)Mb / ( b[j] + t * ( b[i] - b[j] ) ) ) < 0.00001 );
        return (double)Ma / ( a[j] + t * ( a[i] - a[j] ) );
    }
    return max( minDrink( i, Ma, Mb ), minDrink( j, Ma, Mb ) );
}

double tg( int i, int j ) {
    return (double)(b[i] - b[j]) / (a[i] - a[j]);
}

int maxA = 0;

void init(const std::vector<int>& A, const std::vector<int>& B) {
    n = A.size();

    vector <int> order( n );
    for ( int i = 0; i < n; i ++ ) order[i] = i;

    sort( order.begin(), order.end(), [&]( int i, int j ) {
         if ( A[i] != A[j] ) return A[i] < A[j];
         return B[i] > B[j];
    } );
    for ( int i = 0; i < n; i ++ ) {
        a[i] = A[order[i]];
        b[i] = B[order[i]];
        maxA = max( maxA, a[i] );
        minB = min( minB, b[i] );
    }


    for ( int i = 0; i < n; i ++ ) {
        while ( hull.size() >= 2 && ( isBetter( i, hull.back() ) || tg( i, hull.back() ) <= tg( i, hull[hull.size() - 2] ) ) ) {
            candSt[i].push_back( hull.back() );
            candDr[hull.back()].push_back( i );
            hull.pop_back();
        }

        if ( !hull.empty() ) {
            candSt[i].push_back( hull.back() );
            candDr[hull.back()].push_back( i );
        }
        reverse( candSt[i].begin(), candSt[i].end() );
        hull.push_back( i );
    }

}

int findBestPointSmaller( int i, int La ) {
    int st = -1, dr = candSt[i].size();

    while ( st < dr - 1 ) {
        int mij = ( st + dr ) / 2;
        if ( candSt[i][mij] >= La ) dr = mij;
        else st = mij;
    }
    if ( dr < candSt[i].size() ) return candSt[i][dr];
    return -1;
}

int findBestPointBigger( int i, int Ra ) {
    int st = -1, dr = candDr[i].size();

    while ( st < dr - 1 ) {
        int mij = ( st + dr ) / 2;
        if ( candDr[i][mij] <= Ra ) st = mij;
        else dr = mij;
    }

    if ( st != -1 ) return candDr[i][st];
    return -1;
}
double friendDrink(const int Ma, const int Mb, const int La, const int Ra) {
    double ans = 0;
    double st = 0, dr = 1e9;
    int bestPoint = 0;
    for ( int step = 0; step < 100; step ++ ) {
        double mij = ( st + dr ) / 2, maxX = Ma / mij, maxY = Mb / mij;
        int i = 0, j = hull.size();
        while ( i < j - 1 ) {
            int mij = ( i + j ) / 2;
            if ( a[hull[mij]] <= maxX ) i = mij;
            else j = mij;
        }
        if ( a[hull[i]] < La ) dr = mij;
        else if ( a[hull[i]] > Ra ) st = mij;
        else if ( a[hull[i]] <= maxX && b[hull[i]] <= maxY ) {
            st = mij;
            bestPoint = i;
        } else {
            dr = mij;
        }
    }
//    cout << "BEST POINT: " <<  a[hull[bestPoint]] << ' ' << b[hull[bestPoint]] << endl;
//    cout << candSt[2].size() << ' ' << candDr[2].size() << endl;
    ans = minDrink( hull[bestPoint], Ma, Mb );
    int Bestst = findBestPointSmaller( hull[bestPoint], La ), Bestdr = findBestPointBigger( hull[bestPoint], Ra );
//    cout << Bestst << endl;
    if ( Bestst >= 0 ) ans = max( ans, min2Drink( hull[bestPoint], Bestst, Ma, Mb ) );
    if ( Bestdr >= 0 ) ans = max( ans, min2Drink( hull[bestPoint], Bestdr, Ma, Mb ) );
    return ans;
}

double friendDrinkBrut(const int Ma, const int Mb, const int La, const int Ra) {
    double ans = 0;
    for ( int i = 0; i < n; i ++ ) {
        if ( La <= a[i] && a[i] <= Ra ) {
            for ( int j = i; j < n; j ++ ) {
                if ( La <= a[j] && a[j] <= Ra ) {
                    ans = max( ans, min2Drink( i, j, Ma, Mb ) );
                }
            }
        }
    }
    return ans;
}
