#include <vector>
#include <iostream>
#include <algorithm>

#include "soft_drinks.h"

using ld = long double;

const double eps = 1e-10;

const int MAX_N = 1e5 + 5;

int n;
int a[MAX_N + 1], b[MAX_N + 1];
int auxa[MAX_N + 1], auxb[MAX_N + 1];
int auxperm[MAX_N + 1];
int perm[MAX_N + 1];

bool cmp(int x, int y) {
    if (a[x] != a[y])
        return a[x] < a[y];
    return b[x] < b[y];
}

void sorteaza() {
    for (int i = 1; i <= n; i++)
        perm[i] = i;
    std::sort(perm + 1, perm + n + 1, cmp);

    for (int i = 1; i <= n; i++) {
        auxa[i] = a[perm[i]];
        auxb[i] = b[perm[i]];
    }

    for (int i = 1; i <= n; i++) {
        a[i] = auxa[i];
        b[i] = auxb[i];
    }

    //remove a duplicates

    int cnt = 1; auxa[1] = a[1]; auxb[1] = b[1];

    for (int i = 2; i <= n; i++) {
        if (a[i] != auxa[cnt]) {
            cnt++;
            auxa[cnt] = a[i];
            auxb[cnt] = b[i];
        }
    }

    n = cnt;
    for (int i = 1; i <= n; i++) {
        a[i] = auxa[i];
        b[i] = auxb[i];
    }
}

void init(const std::vector<int>& A, const std::vector<int>& B) {
    n = A.size();
    for (int i = 0; i < n; i++) {
        a[i + 1] = A[i];
        b[i + 1] = B[i];
    }
    sorteaza();

    /*for (int i = 1; i <= n; i++)
        std::cout << a[i] << ' ';
    std::cout << '\n';
    for (int i = 1; i <= n; i++)
        std::cout << b[i] << ' ';
    std::cout << '\n';*/
}

bool ok(double volum, int ma, int mb, int la, int ra) {
    for (int i = 1; i <= n; i++) {
        for (int j = i + 1; j <= n; j++) {
            if (a[i] < la || a[i] > ra) continue;
            if (a[j] < la || a[j] > ra) continue;
            if (a[i] <= a[j] && b[i] <= b[j]) continue;

            if (volum * a[i] > ma || volum * b[j] > mb) continue;

            //a[i] < a[j] si b[i] > b[j]

            //std::cout << i << ' ' << j << '\n';




            double v1 = volum, v2 = 0;

            //std::cout << v1 << ' ' << v2 << '\n';


            double schimb = double (ma - volum * a[i]) / double (a[j] - a[i]);

            schimb = std::min(schimb, v1);

            v1 -= schimb; v2 += schimb;
            //std::cout << double (ma - volum * a[i]) << ' ' << v1 << ' ' << v2 << '\n';

            if (v1 >= 0 && v1 * a[i] + v2 * a[j] <= ma && v1 * b[i] + v2 * b[j] <= mb)
                return true;
        }
    }

    for (int i = 1; i <= n; i++) {
        double vmax = 1e9;
        if (a[i]) vmax = std::min(vmax, (double) ma / (double) a[i]);
        if (b[i]) vmax = std::min(vmax, (double) mb / (double) b[i]);
        if (a[i] >= la && a[i] <= ra && vmax >= volum)
            return true;
    }

    return false;
}

double friendDrink(const int Ma, const int Mb, const int La, const int Ra) {
    double st = 0;
    double dr = 1e9 + 5;
    double ans = 0;

    //std::cout << ok(1.86265, Ma, Mb, La, Ra) << '\n';
    //return 0;

    while (dr - st > eps) {
        double mij = (st + dr) / (double) 2;
        bool Ok = ok(mij, Ma, Mb, La, Ra);
        //std::cout << st << ' ' << dr << ' ' << mij << ' ' << Ok << '\n';
        if (ok(mij, Ma, Mb, La, Ra)) {
            ans = std::max(ans, mij);
            st = mij + eps;
        } else {
            dr = mij - eps;
        }
    }

    for (int i = 1; i <= n; i++) {
        double vmax = 1e9 + 5;
        if (a[i]) vmax = std::min(vmax, (double) Ma / (double) a[i]);
        if (b[i]) vmax = std::min(vmax, (double) Mb / (double) b[i]);
        if (a[i] >= La && a[i] <= Ra)
            ans = std::max(ans, vmax);
    }

    return ans;
}

