#include <iostream>
#include <vector>
#include <queue>
#include <algorithm>
#include <set>
#pragma GCC optimize("O2")

using ll = long long;

const int MAX_N = 2e5 + 5;
const int MAX_L = 17;
const int INF = 2e9 + 5;

int n, k;

std::vector<std::pair<int, int>> g[MAX_N + 1];

int sum[MAX_N + 1];
int dpup[MAX_N + 1];
int dp[MAX_N + 1];
int euler[2 * MAX_N + 1];
int tin[MAX_N + 1], tout[MAX_N + 1];
int cnt_euler;
int up[MAX_L + 1][MAX_N + 1];

bool is_ancestor(int x, int y) {
    return tin[x] <= tin[y] && tout[x] >= tout[y];
}

int lca(int x, int y) {
    if (is_ancestor(x, y)) return x;
    if (is_ancestor(y, x)) return y;
    for (int i = MAX_L; i >= 0; i--)
        if (up[i][x] && !is_ancestor(up[i][x], y)) {
            x = up[i][x];
        }
    return up[0][x];
}

int dist(int a, int b) {
    if (a == b) return 0;
    int l = lca(a, b);
    return sum[a] + sum[b] - 2 * sum[l];
}

void dfs1(int node, int dad) {
    dp[node] = 0;
    euler[++cnt_euler] = node;
    tin[node] = cnt_euler;

    up[0][node] = dad;
    for (int i = 1; i <= MAX_L; i++)
        up[i][node] = up[i - 1][up[i - 1][node]];

    for (auto it : g[node])
        if (it.first != dad) {
            g[it.first].erase(std::find(g[it.first].begin(), g[it.first].end(), std::make_pair(node, it.second)));
            sum[it.first] = sum[node] + it.second;
            dfs1(it.first, node);
            dp[node] = std::max(dp[node], dp[it.first] + it.second);
        }

    euler[++cnt_euler] = node;
    tout[node] = cnt_euler;
}

int dp_subarbore[MAX_N + 1];

void dfs2(int node, int dad) {
    if (g[node].empty())
        return;

    std::multiset<int, std::greater<int>> ms;
    for (auto it : g[node]) {
        ms.insert(dp[it.first] + it.second);
    }

    if (ms.size() >= 2) {
        auto it = ms.begin();
        dp_subarbore[node] += *it;
        it++;
        dp_subarbore[node] += *it;
    } else {
        dp_subarbore[node] = dp[node];
    }


    for (auto it : g[node]) {
        ms.erase(ms.find(dp[it.first] + it.second));

        int val = 0;
        if (!ms.empty())
            val = *ms.begin();

        dpup[it.first] = std::max(dpup[node], val) + it.second;

        ms.insert(dp[it.first] + it.second);
    }

    for (auto it : g[node]) {
        dfs2(it.first, node);
    }
}

struct Diametru {
    int x, y, d;
};

Diametru combine(Diametru a, Diametru b) {
    Diametru ans = (a.d > b.d ? a : b);
    for (int node1 : {a.x, a.y})
        for (int node2 : {b.x, b.y}) {
            int nd = dist(node1, node2);
            if (nd > ans.d) {
                ans.d = nd;
                ans.x = node1;
                ans.y = node2;
            }
        }
    return ans;
}

Diametru prefix_euler[2 * MAX_N + 1];
Diametru sufix_euler[2 * MAX_N + 1];

int ans = 2e9;

Diametru get_diametru(int p, int s) {
    if (p > 0 && s <= cnt_euler)
        return combine(prefix_euler[p], sufix_euler[s]);
    if (p > 0)
        return prefix_euler[p];
    if (s <= cnt_euler)
        return sufix_euler[s];
    return {0, 0, 0};
}

void dfs3(int node) {
    for (auto it : g[node]) {
        dfs3(it.first);
        dp_subarbore[node] = std::max(dp_subarbore[node], dp_subarbore[it.first]);
    }
}

void solve() {
    dfs1(1, 0);

    dfs2(1, 0);

    dfs3(1);

    prefix_euler[1] = Diametru {1, 1, 0};
    for (int i = 2; i <= cnt_euler; i++)
        prefix_euler[i] = combine(prefix_euler[i - 1], Diametru{euler[i], euler[i], 0});
    sufix_euler[cnt_euler] = Diametru {1, 1, 0};
    for (int i = cnt_euler - 1; i >= 1; i--)
        sufix_euler[i] = combine(sufix_euler[i + 1], Diametru {euler[i], euler[i], 0});

    for (int node = 1; node <= n; node++) {
        for (auto it : g[node]) {
            int d1 = dp[it.first] + dpup[it.first] - 1;
            int d2 = get_diametru(tin[it.first] - 1, tout[it.first] + 1).d;
            int d3 = dp_subarbore[it.first];
            int mx = std::max({d1, d2, d3});

            if (mx < ans)
                ans = mx;
        }
    }

    std::cout << ans << '\n';

    /*for (int i = 1; i <= n; i++) {
        std::cout << dp_subarbore[i] << ' ';
    }*/
}

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(nullptr);
    std::cin >> n >> k;
    for (int i = 1; i < n; i++) {
        int u, v, w;
        std::cin >> u >> v >> w;
        g[u].push_back(std::make_pair(v, w));
        g[v].push_back(std::make_pair(u, w));
    }
    solve();
    return 0;
}