#include <iostream>
#include <vector>
#include <queue>
#include <algorithm>
#include <set>

using ll = long long;

const int MAX_N = 2e5 + 5;
const int MAX_L = 18;
const int INF = 2e9 + 5;

int n, k;

std::vector<std::pair<int, int>> g[MAX_N + 1];

int sum[MAX_N + 1];
int dpup[MAX_N + 1];
int dp[MAX_N + 1];
int euler[2 * MAX_N + 1];
int tin[MAX_N + 1], tout[MAX_N + 1];
int cnt_euler;
int rmq[MAX_L + 1][2 * MAX_N + 1];
int lvl[MAX_N + 1];
int l2[2 * MAX_N + 1];
int alt_euler[2 * MAX_N + 1];
int cnt_alt_euler;
int alt_tin[MAX_N + 1], alt_tout[MAX_N + 1];

void fa_alt_euler(int node) {
    alt_euler[++cnt_alt_euler] = node;
    alt_tin[node] = cnt_alt_euler;
    for (auto it : g[node]) {
        fa_alt_euler(it.first);
        alt_euler[++cnt_alt_euler] = node;
    }
}

bool is_ancestor(int x, int y) {
    return tin[x] <= tin[y] && tout[x] >= tout[y];
}

static inline int mai_sus(int x, int y) {
    return (lvl[x] < lvl[y] ? x : y);
}

int query(int st, int dr) {
    int l = l2[dr - st + 1];
    return mai_sus(rmq[l][st], rmq[l][dr - (1 << l) + 1]);
}

void build_rmq() {
    for (int i = 1; i <= cnt_alt_euler; i++)
        rmq[0][i] = alt_euler[i];
    for (int l = 1; l <= MAX_L; l++) {
        for (int i = 1; i + (1 << l) - 1 <= cnt_alt_euler; i++) {
            rmq[l][i] = mai_sus(rmq[l - 1][i], rmq[l - 1][i + (1 << (l - 1))]);
        }
    }
}

int lca(int x, int y) {
    if (alt_tin[x] <= alt_tin[y])
        return query(alt_tin[x], alt_tin[y]);
    return query(alt_tin[y], alt_tin[x]);
}

int dist(int a, int b) {
    if (a == b) return 0;
    int l = lca(a, b);
    return sum[a] + sum[b] - 2 * sum[l];
}

void dfs1(int node, int dad) {
    dp[node] = 0;
    euler[++cnt_euler] = node;
    tin[node] = cnt_euler;
    lvl[node] = lvl[dad] + 1;

    for (auto it : g[node])
        if (it.first != dad) {
            g[it.first].erase(std::find(g[it.first].begin(), g[it.first].end(), std::make_pair(node, it.second)));
            sum[it.first] = sum[node] + it.second;
            dfs1(it.first, node);
            dp[node] = std::max(dp[node], dp[it.first] + it.second);
        }

    euler[++cnt_euler] = node;
    tout[node] = cnt_euler;
}

int dp_subarbore[MAX_N + 1];

void dfs2(int node, int dad) {
    if (g[node].empty())
        return;

    std::multiset<int, std::greater<int>> ms;
    for (auto it : g[node]) {
        ms.insert(dp[it.first] + it.second);
    }

    if (ms.size() >= 2) {
        auto it = ms.begin();
        dp_subarbore[node] += *it;
        it++;
        dp_subarbore[node] += *it;
    } else {
        dp_subarbore[node] = dp[node];
    }


    for (auto it : g[node]) {
        ms.erase(ms.find(dp[it.first] + it.second));

        int val = 0;
        if (!ms.empty())
            val = *ms.begin();

        dpup[it.first] = std::max(dpup[node], val) + it.second;

        ms.insert(dp[it.first] + it.second);
    }

    for (auto it : g[node]) {
        dfs2(it.first, node);
    }
}

struct Diametru {
    int x, y, d;
};

Diametru combine(Diametru a, Diametru b) {
    Diametru ans = (a.d > b.d ? a : b);
    for (int node1 : {a.x, a.y})
        for (int node2 : {b.x, b.y}) {
            int nd = dist(node1, node2);
            if (nd > ans.d) {
                ans.d = nd;
                ans.x = node1;
                ans.y = node2;
            }
        }
    return ans;
}

Diametru prefix_euler[2 * MAX_N + 1];
Diametru sufix_euler[2 * MAX_N + 1];

int ans = 2e9;

Diametru get_diametru(int p, int s) {
    if (p > 0 && s <= cnt_euler)
        return combine(prefix_euler[p], sufix_euler[s]);
    if (p > 0)
        return prefix_euler[p];
    if (s <= cnt_euler)
        return sufix_euler[s];
    return {0, 0, 0};
}

void dfs3(int node) {
    for (auto it : g[node]) {
        dfs3(it.first);
        dp_subarbore[node] = std::max(dp_subarbore[node], dp_subarbore[it.first]);
    }
}

void solve() {
    for (int i = 2; i <= 2 * n; i++)
        l2[i] = l2[i >> 1] + 1;

    dfs1(1, 0);

    dfs2(1, 0);

    dfs3(1);

    fa_alt_euler(1);

    build_rmq();

    prefix_euler[1] = Diametru {1, 1, 0};
    for (int i = 2; i <= cnt_euler; i++)
        prefix_euler[i] = combine(prefix_euler[i - 1], Diametru{euler[i], euler[i], 0});
    sufix_euler[cnt_euler] = Diametru {1, 1, 0};
    for (int i = cnt_euler - 1; i >= 1; i--)
        sufix_euler[i] = combine(sufix_euler[i + 1], Diametru {euler[i], euler[i], 0});

    for (int node = 1; node <= n; node++) {
        for (auto it : g[node]) {
            int d1 = dp[it.first] + dpup[it.first] - 1;
            int d2 = get_diametru(tin[it.first] - 1, tout[it.first] + 1).d;
            int d3 = dp_subarbore[it.first];
            int mx = std::max({d1, d2, d3});

            if (mx < ans)
                ans = mx;
        }
    }

    std::cout << ans << '\n';

    /*for (int i = 1; i <= cnt_euler; i++)
        std::cout << euler[i] << ' ';
    std::cout << '\n';
    for (int l = 0; l <= 3; l++) {
        for (int i = 1; i <= cnt_euler; i++)
            std::cout << rmq[l][i] << ' ';
        std::cout << '\n';
    }

    for (int i = 1; i <= n; i++) {
        for (int j = 1; j <= n; j++) {
            std::cout << i << ' ' << j << ' ' << lca(i, j) << '\n';
        }
    }*/

    /*for (int i = 1; i <= n; i++) {
        std::cout << dp_subarbore[i] << ' ';
    }*/
}

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(nullptr);
    std::cin >> n >> k;
    for (int i = 1; i < n; i++) {
        int u, v, w;
        std::cin >> u >> v >> w;
        g[u].push_back(std::make_pair(v, w));
        g[v].push_back(std::make_pair(u, w));
    }
    solve();
    return 0;
}
