#include <iostream>
#include <vector>
#include <cassert>
#include <algorithm>
#include <random>
#include "cycles.h"

using namespace std;

mt19937 rnd(1329);

void sortPermutation(int N) {
	int cur = performSwap(0, 1);
	if (cur == N) return;
	int x;
	for (int i = 1; i < N; ++i) {
		int t = performSwap(i, i + 1);
		if (t == N) return;
		if (t > cur) {
			x = i + 1;
			cur = t;
			break;
		}
		cur = t;
	}
	for (int i = 0; i < N; ++i) {
		int t = performSwap(i, x);
		if (t == N) return;
		if (t < cur) {
			performSwap(i, x);
			if (i == 0) {
				for (int j = 0; j < x - 1; ++j) {
					performSwap(j, x - 1);
				}
				return;
			}
			if (i) performSwap(i - 1, x - 1);
			for (int j = i; j < x; ++j) {
				performSwap(i - 1, j);
			}
			return;
		}
		cur = t;
	}
}

#ifdef LOCAL
static int64_t cnt = 0;
static std::vector<int> perm;
static int64_t n;

static int findPermCount() {
	std::vector<bool> used(n, false);
	int64_t cntc = 0;
	for (int64_t i = 0; i < n; i ++) {
		if (used[i]) { continue; }
		int64_t curr = i; cntc ++;
		while (!used[curr]) {
			used[curr] = true;
			curr = perm[curr];
		}
	}
    return cntc;
}

int performSwap(int i, int j) {
	++cnt;

    bool valid_swap = true;
    valid_swap &= (0 <= i && i < n);
    valid_swap &= (0 <= j && j < n);
    valid_swap &= (i != j);

    if (!valid_swap) {
        std::cout << "WRONG. Your program made an invalid swap: (" << i << ", " << j << ")" << std::endl;
        exit(0);  
    }

	std::swap(perm[i], perm[j]);
	return findPermCount();
}

int main() {
	freopen("in.txt", "r", stdin);
	freopen("out.txt", "w", stdout);
	std::cin >> n;
	perm.resize(n);
	for (int64_t i = 0; i < n; i ++) { std::cin >> perm[i]; }

	sortPermutation(n);
	if (findPermCount() != n) {
		std::cout << "WRONG. The final permutation is not sorted." << std::endl;
		for (auto e : perm) cout << e << ' ';
	} else {
		std::cout << "CORRECT with " << cnt << " swaps" << std::endl; 
	}
	return 0;
}
#endif