#include <iostream>
#include <vector>
#include <cassert>
#include <algorithm>
#include <random>
#include "cycles.h"

using namespace std;

mt19937 rnd(1329);

int cur;

vector<int> get_cycle(vector<int> ord) {
	shuffle(ord.begin(), ord.end(), rnd);
	vector<int> cycle;
	cycle.push_back(ord[0]);
	int n = ord.size();
	for (int i = 1; i < n; ++i) {
		int t = performSwap(ord[0], ord[i]);
		if (t > cur) {
			cycle.push_back(ord[i]);
		}
		performSwap(ord[0], ord[i]);
	}
	return cycle;
}

vector<vector<int>> split_cycle(vector<int> cycle, int k) {
	shuffle(cycle.begin(), cycle.end(), rnd);
	cur = performSwap(cycle[0], cycle[1]);
	for (int i = 2; i < k; ++i) {
		for (int j = 0; j < i; ++j) {
			int t = performSwap(cycle[i], cycle[j]);
			if (t > cur) {
				cur = t;
				break;
			}
			performSwap(cycle[i], cycle[j]);
		}
	}
	vector<vector<int>> cycles(k);
	for (int i = 0; i < k; ++i) cycles[i].push_back(cycle[i]);
	for (int i = k; i < cycle.size(); ++i) {
		bool ok = 0;
		for (int j = 0; j < k - 1; ++j) {
			int t = performSwap(cycle[i], cycle[j]);
			if (t > cur) {
				performSwap(cycle[i], cycle[j]);
				cycles[j].push_back(cycle[i]);
				ok = 1;
				break;
			}
			performSwap(cycle[i], cycle[j]);
		}
		if (!ok) cycles[k - 1].push_back(cycle[i]);
	}
	return cycles;
}

void solve_for_cycle(vector<int> cycle) {
	if (cycle.size() == 0) return;
	if (cycle.size() == 1) return;
	if (cycle.size() == 2) {
		int t = performSwap(cycle[0], cycle[1]);
		if (t < cur) cur = performSwap(cycle[0], cycle[1]);
		else cur = t;
		return;
	}
	int k = 2;
	if (cycle.size() >= 50) k = 3;
	vector<vector<int>> t = split_cycle(cycle, k);
	for (auto e : t) solve_for_cycle(e);
	// shuffle(cycle.begin(), cycle.end(), rnd);
	// int i = 0, j = 1;
	// // int i = rnd() % cycle.size();
	// // int j = rnd() % cycle.size();
	// // while (i == j) j = rnd() % cycle.size();
	// cur = performSwap(cycle[i], cycle[j]);
	// vector<int> c1, c2;
	// c1.push_back(cycle[0]);
	// for (int i = 1; i < cycle.size(); ++i) {
	// 	int t = performSwap(cycle[0], cycle[i]);
	// 	if (t > cur) c1.push_back(cycle[i]);
	// 	else c2.push_back(cycle[i]);
	// 	performSwap(cycle[0], cycle[i]);
	// }
	// solve_for_cycle(c1);
	// solve_for_cycle(c2);
}


void sortPermutation(int N) {
	cur = performSwap(0, 1);
	vector<bool> used(N);
	int curkek = N;
	while (curkek > 0) {
		vector<int> lol;
		for (int i = 0; i < N; ++i) if (!used[i]) lol.push_back(i);
		vector<int> c = get_cycle(lol);
		solve_for_cycle(c);
		curkek -= c.size();
		for (auto e : c) used[e] = 1;
	}
}

#ifdef LOCAL
static int64_t cnt = 0;
static std::vector<int> perm;
static int64_t n;

static int findPermCount() {
	std::vector<bool> used(n, false);
	int64_t cntc = 0;
	for (int64_t i = 0; i < n; i ++) {
		if (used[i]) { continue; }
		int64_t curr = i; cntc ++;
		while (!used[curr]) {
			used[curr] = true;
			curr = perm[curr];
		}
	}
    return cntc;
}

int performSwap(int i, int j) {
	++cnt;

    bool valid_swap = true;
    valid_swap &= (0 <= i && i < n);
    valid_swap &= (0 <= j && j < n);
    valid_swap &= (i != j);

    if (!valid_swap) {
        std::cout << "WRONG. Your program made an invalid swap: (" << i << ", " << j << ")" << std::endl;
        exit(0);  
    }

	std::swap(perm[i], perm[j]);
	return findPermCount();
}

int main() {
	freopen("in.txt", "r", stdin);
	freopen("out.txt", "w", stdout);
	std::cin >> n;
	perm.resize(n);
	for (int64_t i = 0; i < n; i ++) { std::cin >> perm[i]; }

	sortPermutation(n);
	if (findPermCount() != n) {
		std::cout << "WRONG. The final permutation is not sorted." << std::endl;
		for (auto e : perm) cout << e << ' ';
	} else {
		std::cout << "CORRECT with " << cnt << " swaps" << std::endl; 
	}
	return 0;
}
#endif