﻿#include <iostream>
#include <vector>
#include <algorithm>
#include <string>
#include <set>
#include <map>
#include <cmath>
#include <array>
#include <random>
#include <queue>


#include "soft_drinks.h"
using namespace std;
typedef long long ll;
typedef long double ld;
const ld EPS = 1e-9;
int sign(ld x) {
	if (abs(x) < EPS) return 0;
	if (x > 0) return 1;
	return -1;
}
struct Point {
	ld x, y;
	Point() : x(0), y(0) {}
	Point(int x, int y) : x(x), y(y) {}
	Point operator+(const Point& a) const {
		return Point(x + a.x, y + a.y);
	}
	Point operator-(const Point& a) const {
		return Point(x - a.x, y - a.y);
	}
	Point operator*(ld k) const {
		return Point(x * k, y * k);
	}
	ld operator*(const Point& a) const {
		return x * a.x + y * a.y;
	}
	ld operator%(const Point& a) const {
		return x * a.y - y * a.x;
	}
	ld len() const {
		return sqrt(x * x + y * y);
	}
	int znak() const {
		if (sign(y) != 0) return sign(y);
		return sign(x);
	}
};
int n;
vector<Point> p, psrt;
vector<ld> mnx, mny;
vector<Point> conv;
vector<pair<Point, int>> vc;
vector<Point> ConvexHull(vector<Point> p) {
	if (p.size() == 0) {
		return vector<Point>{};
	}
	swap(p[0], *min_element(p.begin(), p.end(), [](Point a, Point b) {
		if (sign(a.x - b.x) != 0) return a.x < b.x;
		return a.y < b.y;
		}));
	sort(p.begin() + 1, p.end(), [&](Point a, Point b) {
		a = a - p[0];
		b = b - p[0];
		if (sign(a % b) != 0) return sign(a % b) > 0;
		return a.len() < b.len();
		});
	if (p.size() == 1) return {p[0]};
	vector<Point> st = {p[0], p[1]};
	for (int i = 2; i < p.size(); i++) {
		while (st.size() >= 3) {
			Point f = st.back(), g = st[st.size() - 2];
			if (sign((f - g) % (p[i] - g)) <= 0) {
				st.pop_back();
			} else {
				break;
			}
		}
		st.push_back(p[i]);
	}
	return st;
}
bool comp(Point a, Point b) {
	if (a.znak() != b.znak()) return a.znak() > b.znak();
	if (sign(a % b) != 0) return sign(a % b) > 0;
	return false;
}
bool camp(pair<Point, int> aa, pair<Point, int> bb) {
	Point a = aa.first, b = bb.first;
	if (a.znak() != b.znak()) return a.znak() > b.znak();
	if (sign(a % b) != 0) return sign(a % b) > 0;
	return false;
}
void realinit(vector<Point> p) {
	n = p.size();
	conv = ConvexHull(p);
	for (int i = 0; i < conv.size(); i++) {
		int j = (i + 1) % conv.size();
		vc.push_back({conv[j] - conv[i], j});
	}
	sort(vc.begin(), vc.end(), camp);
	psrt = p;
	if (n >= 2) {
		sort(psrt.begin(), psrt.end(), comp);
		mny.resize(n);
		mny[n - 1] = psrt[n - 1].y;
		for (int i = n - 2; i >= 0; i--)
			mny[i] = min(mny[i + 1], psrt[i].y);
		mnx.resize(n);
		mnx[0] = psrt[0].x;
		for (int i = 1; i < n; i++)
			mnx[i] = min(mnx[i - 1], psrt[i].x);
	}
}
void init(const vector<int>& a, const vector<int>& b) {
	set<pair<int, int>> un;
	for (int i = 0; i < a.size(); i++) {
		if (un.find({a[i], b[i]}) == un.end()) {
			p.push_back(Point(a[i], b[i]));
		}
		un.insert({a[i], b[i]});
	}
	realinit(p);
}
pair<int, int> kass(Point v) {
	int r = lower_bound(vc.begin(), vc.end(), pair<Point, int>{v, 0},
		camp) - vc.begin() - 1;
	if (r < 0) r += vc.size();
	r = vc[r].second;
	int l = lower_bound(vc.begin(), vc.end(), pair<Point, int>{v * -1.0, 0},
		camp) - vc.begin() - 1;
	if (l < 0) l += vc.size();
	l = vc[l].second;
	return {l, r};
}
double friendDrink(const int Ma, const int Mb, const int La, const int Ra) {
	Point v = Point(Ma, Mb);
	ld l = La, r = Ra;
	ld res = 0;
	vector<Point> af, bf;
	for (int i = 0; i < n; i++) {
		if (sign(p[i].x - l) >= 0 && sign(r - p[i].x) >= 0) {
			ld t = 1e18;
			if (p[i].x > EPS) {
				t = min(t, v.x / p[i].x);
			}
			if (p[i].y > EPS) {
				t = min(t, v.y / p[i].y);
			}
			res = max(res, t);
			if (sign(p[i] % v) >= 0) bf.push_back(p[i]);
			else af.push_back(p[i]);
		}
	}
	if (af.size() > 0 && bf.size() > 0) {
		Point bp, ap;
		ap = *min_element(af.begin(), af.end(), [&](Point r, Point s) {
			return sign((r * v) - (s * v)) < 0;
			});
		bp = *min_element(af.begin(), af.end(), [&](Point r, Point s) {
			return sign((r * v) - (s * v)) < 0;
			});
		for (auto w : bf) {
			Point a = Point(), b = ap, u = w - ap;
			ld ch = (a - b) % v, zn = u % v;
			if (sign(zn) != 0) {
				ld q = ch / zn;
				if (sign(q) >= 0 && sign(1 - q) >= 0) {
					ld t = ((b - a) % u) / (v % u);
					res = max(res, 1 / t);
				}
			}
		}
		for (auto w : af) {
			Point a = Point(), b = bp, u = w - bp;
			ld ch = (a - b) % v, zn = u % v;
			if (sign(zn) != 0) {
				ld q = ch / zn;
				if (sign(q) >= 0 && sign(1 - q) >= 0) {
					ld t = ((b - a) % u) / (v % u);
					res = max(res, 1 / t);
				}
			}
		}
	}
	return res;
}
//int main() {
//	ios_base::sync_with_stdio(0);
//	cin.tie(0);
//	cout.tie(0);
//
//	return 0;
//}