#include <vector>
#include <algorithm>

using namespace std;

constexpr int INF = 1e9 + 7;

struct Line {
    double k, b;

    Line(double _k, double _b) {
        k = _k, b = _b;
    }

    double operator()(double x) {
        return k * x + b;
    }
};

double intersectiont(Line l1, Line l2) {
    // l1.k * x + l1.b = l2.k * x + l2.b
    // (l1.k - l2.k) * x = l2.b - l1.b
    return (l2.b - l1.b) / (l1.k - l2.k);
}

struct CHT {
    vector<Line> lines;
    vector<double> points;

    CHT() {

    }

    void addLine(Line l) {
        if (!lines.empty() && l.k == lines.back().k) {
            return;
        }
        while (lines.size() > 1 && intersectiont(lines.rbegin()[1], l) <= points.back()) {
            lines.pop_back();
            points.pop_back();
        }
        if (!lines.empty()) {
            points.push_back(intersectiont(lines.back(), l));
        }
        lines.push_back(l);
    }

    double getAns(int Ma, int Mb, double ll) {
        double ans = INF;
        for (int i = 0; i < (int)points.size(); ++i) {
            if (points[i] >= ll) {
                ans = min(ans, points[i] * Ma + lines[i](points[i]) * Mb);
                if (i == 0 || points[i - 1] < ll) {
                    ans = min(ans, ll * Ma + lines[i](ll) * Mb);
                }
            }
        }
        if (points.empty() || points.back() < ll) {
            ans = min(ans, ll * Ma + lines.back()(ll) * Mb);
        }
        return ans;
    }
};

int n;

vector<int> a, b;

void init(const vector<int>& A, const vector<int>& B) {
    n = A.size();
    a = A, b = B;
}

double friendDrink(const int Ma, const int Mb, const int La, const int Ra) {
    vector<pair<int, int>> nw;
    vector<Line> lines;
    double ll = 0;
    for (int i = 0; i < n; ++i) {
        if (La <= a[i] && a[i] <= Ra) {
            if (b[i] == 0) {
                ll = max(ll, 1 / (double)a[i]);
            } else {
                lines.emplace_back(-a[i] / (double)b[i], 1 / (double)b[i]);
            }
        }
    }
    lines.emplace_back(0, 0);
    sort(lines.begin(), lines.end(), [&](Line l1, Line l2) {
        if (l1.k != l2.k) {
            return l1.k < l2.k;
        }
        return l1.b > l2.b;
    });
    CHT cht;
    for (auto line : lines) {
        cht.addLine(line);
    }
    return cht.getAns(Ma, Mb, ll);
}
