#include <vector>
#include <algorithm>
#include <assert.h>

using namespace std;

constexpr int INF = 2e9 + 7;

constexpr double eps = 1e-9;

struct Line {
    double k, b;

    Line(double _k, double _b) {
        k = _k, b = _b;
    }

    double operator()(double x) {
        return k * x + b;
    }
};

double intersectiont(Line l1, Line l2) {
    return (l2.b - l1.b) / (l1.k - l2.k);
}

struct CHT {
    vector<Line> lines;
    vector<double> points;

    CHT() {

    }

    void addLine(Line l) {
        if (!lines.empty() && l.k == lines.back().k) {
            return;
        }
        while (lines.size() > 1 && intersectiont(lines.rbegin()[1], l) <= points.back()) {
            lines.pop_back();
            points.pop_back();
        }
        if (!lines.empty()) {
            points.push_back(intersectiont(lines.back(), l));
        }
        lines.push_back(l);
    }

    double getAns(double Ma, double Mb, double ll) {
        double ans = INF;
        for (int i = 0; i < (int)points.size(); ++i) {
            if (points[i] >= ll) {
                ans = min(ans, points[i] * Ma + lines[i](points[i]) * Mb);
                if (i == 0 || points[i - 1] < ll) {
                    ans = min(ans, ll * Ma + lines[i](ll) * Mb);
                }
            }
        }
        if (points.empty() || points.back() < ll) {
            ans = min(ans, ll * Ma + lines.back()(ll) * Mb);
        }
        return ans;
    }

    vector<Line> buildAns(double ll) {
        vector<Line> linesAns;
        for (int i = 0; i < (int)points.size(); ++i) {
            if (points[i] >= ll) {
                linesAns.emplace_back(lines[i](points[i]), points[i]);
                if (i == 0 || points[i - 1] < ll) {
                    linesAns.emplace_back(lines[i](ll), ll);
                }
            }
        }
        if (points.empty() || points.back() < ll) {
            linesAns.emplace_back(lines.back()(ll), ll);
        }
        return linesAns;
    }

    double operator()(double x) {
        int pos = lower_bound(points.begin(), points.end(), x) - points.begin();
        return lines[pos](x);
    }
};

int n;

vector<int> a, b;

vector<pair<int, int>> vals;

double ll = 0;

CHT cht;

CHT chtAns;

int mn_b = INF;

void init(const vector<int>& A, const vector<int>& B) {
    n = A.size();
    a = A, b = B;
    for (int i = 0; i < n; ++i) {
        if (a[i] == 0) {
            mn_b = min(mn_b, b[i]);
        }
        vals.emplace_back(a[i], b[i]);
    }
    sort(vals.begin(), vals.end());
    vector<Line> lines;
    for (int i = 0; i < n; ++i) {
        if (b[i] == 0) {
            ll = max(ll, 1 / (double)a[i]);
        } else {
            lines.emplace_back(-a[i] / (double)b[i], 1 / (double)b[i]);
        }
    }
    lines.emplace_back(0, 0);
    sort(lines.begin(), lines.end(), [&](Line l1, Line l2) {
        if (l1.k != l2.k) {
            return l1.k < l2.k;
        }
        return l1.b > l2.b;
    });
    for (auto line : lines) {
        cht.addLine(line);
    }
    vector<Line> linesAns = cht.buildAns(ll);
    sort(linesAns.begin(), linesAns.end(), [&](Line l1, Line l2) {
        if (l1.k != l2.k) {
            return l1.k > l2.k;
        }
        return l1.b < l2.b;
    });
    for (auto line : linesAns) {
        chtAns.addLine(line);
    }
}

double friendDrink(const int Ma, const int Mb, const int La, const int Ra) {
    double l = ll, r = INF;
    while (r - l > eps) {
        double m1 = l + (r - l) / 3, m2 = l + (r - l) / 3 * 2;
        double ans1 = m1 * Ma + cht(m1) * Mb, ans2 = m2 * Ma + cht(m2) * Mb;
        if (ans1 < ans2) {
            r = m2;
        } else {
            l = m1;
        }
    }
    return l * Ma + cht(l) * Mb;
    if (Ma != 0) {
        return chtAns(Mb / (double)Ma) * Ma;
    }
    return Mb / (double)mn_b;
}
