#include <iostream>
#include <vector>
#include <cassert>
#include <algorithm>
#include <random>
#ifndef _DEBUG
#include "cycles.h"
#endif
#ifdef _DEBUG
int performSwap(int x, int y);
#endif
using namespace std;
mt19937 rng(31);

int x;

void rec(vector<int>& v) {
	if (v.size() == 1) return;
	int i = v[rng() % v.size()];
	int j = rng() % v.size();
	while (v[j] == i) {
		j = rng() % v.size();
	}
	j = v[j];
	x = performSwap(i, j);
	vector<int> a, b;
	for (int q = 0; q < v.size(); ++q) {
		if (v[q] == i) {
			a.push_back(v[q]);
			continue;
		}
		if (v[q] == j) {
			b.push_back(v[q]);
			continue;
		}
		int y = performSwap(v[q], i);
		if (y > x) {
			a.push_back(v[q]);
		}
		else {
			b.push_back(v[q]);
		}
		performSwap(v[q], i);
	}
	rec(a);
	rec(b);
}

void sortPermutation(int n) {
	x = performSwap(0, 1);
	for (int i = 1; i < n; ++i) {
		int y = performSwap(0, i);
		if (y > x) {
			performSwap(0, i);
		}
		else {
			x = y;
		}
	}
	vector<int> v(n);
	for (int i = 0; i < n; ++i) {
		v[i] = i;
	}
	rec(v);
}

#ifdef _DEBUG
static int64_t cnt = 0;
static std::vector<int> perm;
static int64_t n;

static int findPermCount() {
	std::vector<bool> used(n, false);
	int64_t cntc = 0;
	for (int64_t i = 0; i < n; i++) {
		if (used[i]) { continue; }
		int64_t curr = i; cntc++;
		while (!used[curr]) {
			used[curr] = true;
			curr = perm[curr];
		}
	}
	return cntc;
}

int performSwap(int i, int j) {
	++cnt;

	bool valid_swap = true;
	valid_swap &= (0 <= i && i < n);
	valid_swap &= (0 <= j && j < n);
	valid_swap &= (i != j);

	if (!valid_swap) {
		std::cout << "WRONG. Your program made an invalid swap: (" << i << ", " << j << ")" << std::endl;
		exit(0);
	}

	std::swap(perm[i], perm[j]);
	return findPermCount();
}

int main() {
	std::cin >> n;
	perm.resize(n);
	for (int64_t i = 0; i < n; i++) { std::cin >> perm[i]; }

	sortPermutation(n);
	if (findPermCount() != n) {
		std::cout << "WRONG. The final permutation is not sorted." << std::endl;
	}
	else {
		std::cout << "CORRECT with " << cnt << " swaps" << std::endl;
	}
	return 0;
}
#endif