#include <bits/stdc++.h>

using namespace std;

//#define int ll
using ll = long long;
using vi = vector<int>;
using ii = pair<int, int>;
using pl = pair<ll, ll>;
using vll = vector<ll>;
using ld = long double;
using pct = pair<ld, ld>;

const ld INF = 1e9;
const ld EPS = 1e-8;

int n, len;
vector<pct> V, ConvHull;

ld cross(pct a, pct b) {
    return a.first * b.second - a.second * b.first;
}

ld det(pct a, pct b, pct c) {
    return cross({a.first - c.first, a.second - c.second}, 
        {b.first - c.first, b.second - c.second});
}

void init(const vi &A, const vi &B) {
    n = int(A.size());
    for(int i = 0; i < n; ++i) {
        V.push_back({A[i], B[i]});
    }
    sort(V.begin(), V.end());
    auto good = [&](pct a, pct b, pct c) {
        return det(a, b, c) >= 0;
    };
    for (auto it : V) {
        while (ConvHull.size() >= 2 
            && !good(ConvHull.end()[-2], ConvHull.end()[-1], it)) {
            ConvHull.pop_back();
        }
        ConvHull.push_back(it);
    }
    len = (int)ConvHull.size();
//    printf("Convex Hull:");
//    for(auto [x, y] : ConvHull)
//        printf("  %lf %lf\n", (double)x, (double)y);
}

struct fun {
    ld a, b; /// a x + b

    fun(ld a0 = 0., ld b0 = 0.) : a(a0), b(b0) {}
    fun(pct p1, pct p2) {
        assert(abs(p1.first - p2.first) >= EPS); /// TODO: check later, maybe use a = INF
        a = (p2.second - p1.second) / ld(p1.second - p2.second);
        b = p1.second - a * p1.first;
    }

    ld operator()(ld x) { return a * x + b; }
};

ld intersect(fun f1, fun f2) {
    if(abs(f1.a - f2.a) < EPS) return 0;
    return (f1.b - f2.b) / (f2.a - f1.a);
}

ld val_intersect(fun f1, fun f2) {
    return f1(intersect(f1, f2));
}

ld get_val(pct p1, pct p2, int Ma, int Mb) {
    auto [x1, y1] = p1; auto [x2, y2] = p2;
    fun f1{ld(x1 - x2) / Ma, ld(x2) / Ma},
        f2{ld(y1 - y2) / Mb, ld(y2) / Mb};

    auto get_f = [&](ld x) { /// functii de lambda !!! (interpolare intre x1 si x2)
        /// 0 <= x <= 1 !!
        return max(f1(x), f2(x));
    };
    ld re = INF;

    if(abs(f1.a - f2.a) > EPS) {
        ld x_i = intersect(f1, f2);
        if(x_i > 0. && x_i < 1.)
            re = get_f(x_i);
    }
    re = min({re, get_f(0.), get_f(1.)});

    return 1. / re;
}

ld get_val(pct p1, int Ma, int Mb) {
    return min(ld(Ma) / p1.first, ld(Mb) / p1.second);
}

pct intersect_hull(int p) {
    int st = 0, dr = len - 1, mij;
    while(st < dr) {
        mij = (st + dr + 1) / 2;
        auto [x, y] = ConvHull[mij];
        if(x <= p) st = mij;
        else dr = mij - 1;
    }
    ///undeva intre st si st + 1
    auto [x1, y1] = ConvHull[st];  
    auto [x2, y2] = ConvHull[st + 1];  
    return {p, y1 + (y2 - y1) / ld(x2 - x1) * (p - x1)};
}



double friendDrink(int Ma, int Mb, int La, int Ra) {
    ld re = 0;
    for(int i = 0; i < n; ++i) {
        auto [x1, y1] = V[i];
        if(x1 < La || x1 > Ra) continue;
        re = max(re, get_val(V[i], Ma, Mb));
        for(int j = i + 1; j < n; ++j) {
            auto [x2, y2] = V[j];
            if(x2 < La || x2 > Ra) continue;
            re = max(re, get_val(V[i], V[j], Ma, Mb));
        }
    }
    return re;
}

//int main() {
////	std::ios_base::sync_with_stdio(false); std::cin.tie(NULL);
//
//	int n, q;
//	std::cin >> n >> q;
//
//	std::vector<int> A(n), B(n);
//	for (int i = 0; i < n; i ++) {
//		std::cin >> A[i] >> B[i];
//	}
//	init(A, B);
//
//	for (int i = 0; i < q; i ++) {
//		int a, b, l, r;
//		std::cin >> a >> b >> l >> r;
//		const auto ret = friendDrink(a, b, l, r);
//		std::cout << fixed << setprecision(9) << ret << "\n";
//	}
//
//	return 0;
//}
