#include "soft_drinks.h"

#include <vector>
#include <algorithm>

#include <stdio.h>
#include <assert.h>

namespace __Sol__ {
  using ll = long long;
  constexpr double EPS = 1e-8;

  int sgn( double x ) {
    return (x > EPS) - (x < -EPS);
  }

  int n;
  std::vector<int> A, B;

#define cvec const std::vector<int> &
  void init( cvec __A, cvec __B ) {
    A = __A;
    B = __B;
    n = (int)A.size();
  }

  double solve2( int maxX, int maxY, std::pair<int, int> vec1, std::pair<int, int> vec2 ) {
    // either one -- hits boundary
    // or two -- gets both

    double ret = 0;
    // division by 0 should not matter that much
    ret = std::max( ret, std::min( maxX / (double)vec1.first, maxY / (double)vec1.second ) );
    ret = std::max( ret, std::min( maxX / (double)vec2.first, maxY / (double)vec2.second ) );

    double mat[2][2] = {
      { vec1.first, vec1.second },
      { vec2.first, vec2.second }
    };

    double det = mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0];
    if( sgn(det) == 0 )
      return ret;

    // luam adjuncta
    std::swap( mat[0][0], mat[1][1] );
    std::swap( mat[1][0], mat[0][1] );
    mat[1][0] *= -1;
    mat[0][1] *= -1;

    // impartim la det ca sa gasim inversa
    mat[0][0] /= det;
    mat[0][1] /= det;
    mat[1][0] /= det;
    mat[1][1] /= det;

    double a1 = mat[0][0] * maxX + mat[0][1] * maxY;
    double a2 = mat[1][0] * maxX + mat[1][1] * maxY;

    //assert( sgn(vec1.first * a1 + vec2.first * a2 - maxX) == 0 );
    //assert( sgn(vec1.second * a1 + vec2.second * a2 - maxY) == 0 );

    if( a1 < 0 || a2 < 0 )
      return ret;

    ret = std::max( ret, a1 + a2 );
    return ret;
  }

  double query( int maxX, int maxY, int minA, int maxA ) {
    std::vector<std::pair<int, int>> variante;
    for( int i = 0; i < n; i++ )
      if( minA <= A[i] && A[i] <= maxA )
        variante.emplace_back( A[i], B[i] );

    double ret = 0;
    for( int i = 0; i < (int)variante.size(); i++ )
      for( int j = i; j < (int)variante.size(); j++ )
        ret = std::max( ret, solve2( maxX, maxY, variante[i], variante[j] ) );

    return ret;
  }
}

//void init(const std::vector<int>& A, const std::vector<int>& B) { __Sol__::init( A, B ); }
//double friendDrink(const int Ma, const int Mb, const int La, const int Ra) { return __Sol__::query( Ma, Mb, La, Ra ); }
