#include "soft_drinks.h"

#include <vector>
#include <algorithm>

#include <stdio.h>
#include <assert.h>

namespace __Sol__ {
  using ll = long long;
  constexpr double EPS = 1e-8;

  int sgn( double x ) {
    return (x > EPS) - (x < -EPS);
  }

  int n;
  std::vector<int> A, B;

  double solve2( int maxX, int maxY, std::pair<int, int> vec1, std::pair<int, int> vec2 ) {
    // either one -- hits boundary
    // or two -- gets both

    double ret = 0;
    // division by 0 should not matter that much
    //ret = std::max( ret, std::min( maxX / (double)vec1.first, maxY / (double)vec1.second ) );
    //ret = std::max( ret, std::min( maxX / (double)vec2.first, maxY / (double)vec2.second ) );

    double mat[2][2] = {
      { vec1.first, vec1.second },
      { vec2.first, vec2.second }
    };

    double det = mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0];
    if( sgn(det) == 0 )
      return ret;

    // luam adjuncta
    std::swap( mat[0][0], mat[1][1] );
    std::swap( mat[1][0], mat[0][1] );
    mat[1][0] *= -1;
    mat[0][1] *= -1;

    // impartim la det ca sa gasim inversa
    mat[0][0] /= det;
    mat[0][1] /= det;
    mat[1][0] /= det;
    mat[1][1] /= det;

    double a1 = mat[0][0] * maxX + mat[0][1] * maxY;
    double a2 = mat[1][0] * maxX + mat[1][1] * maxY;

    //assert( sgn(vec1.first * a1 + vec2.first * a2 - maxX) == 0 );
    //assert( sgn(vec1.second * a1 + vec2.second * a2 - maxY) == 0 );

    if( a1 < 0 || a2 < 0 )
      return ret;

    ret = std::max( ret, a1 + a2 );
    return ret;
  }

#define cvec const std::vector<int> &
  std::vector<std::pair<int, int>> points;
  void init( cvec __A, cvec __B ) {
    A = __A;
    B = __B;
    n = (int)A.size();

    points.clear();
    for( int i = 0; i < n; i++ )
      points.emplace_back( A[i], B[i] );

    {// yeet useless vectors (for now only (0, inf) subtask)
      std::vector<std::pair<int, int>> good;
      std::sort( points.begin(), points.end(), std::greater<std::pair<int, int>>() );
      for( auto [x, y]: points ){
        while( !good.empty() && good.back().second >= y )
          good.pop_back();
        good.emplace_back( x, y );
      }

      std::reverse( good.begin(), good.end() );
      points = good;
    }
  }

  double query( int maxX, int maxY, int minA, int maxA ) {
    assert( minA == 0 && maxA == 1000000000 );

    int n = (int)points.size();
    int idx = std::lower_bound( points.begin(), points.end(), std::pair<int, int>(maxX, maxY), []( const std::pair<int, int> &a, const std::pair<int, int> &b ) -> bool {
      return a.first * (ll)b.second < b.first * (ll)a.second;
    } ) - points.begin();

    // should only be first and last, for now i only care about not being wrong abount 2-vector solutions
    double ret = 0;
    for( auto [x, y]: points ){
      ret = std::max( ret, std::min( maxX / (double)x, maxY / (double)y ) );
    }

    if( idx == 0 || idx >= n )
      return ret;

    ret = std::max( ret, solve2( maxX, maxY, points[idx - 1], points[idx] ) );

    // only (0, inf) subtask now...
    return ret;

    /*std::vector<std::pair<int, int>> variante;
    for( int i = 0; i < n; i++ )
      if( minA <= A[i] && A[i] <= maxA )
        variante.emplace_back( A[i], B[i] );

    double ret = 0;
    for( int [x, y]: variante ){
      ret = std::max( ret, std::min( maxX / (double)x, maxY / (double)y ) );
    }

    // for( int i = 0; i < (int)variante.size(); i++ )
    //   for( int j = i; j < (int)variante.size(); j++ )
    //     ret = std::max( ret, solve2( maxX, maxY, variante[i], variante[j] ) );

    return ret;
    */
  }
}

void init(const std::vector<int>& A, const std::vector<int>& B) { __Sol__::init( A, B ); }
double friendDrink(const int Ma, const int Mb, const int La, const int Ra) { return __Sol__::query( Ma, Mb, La, Ra ); }
