#include "soft_drinks.h"

#include <vector>
#include <algorithm>

#include <stdio.h>
#include <assert.h>

namespace __Sol__ {
  using ll = long long;
  constexpr double EPS = 1e-8;

  int sgn( double x ) {
    return (x > EPS) - (x < -EPS);
  }

  int n;
  std::vector<int> A, B;

  double solve2( int maxX, int maxY, std::pair<int, int> vec1, std::pair<int, int> vec2 ) {
    // either one -- hits boundary
    // or two -- gets both

    double ret = 0;
    // division by 0 should not matter that much
    ret = std::max( ret, std::min( maxX / (double)vec1.first, maxY / (double)vec1.second ) );
    ret = std::max( ret, std::min( maxX / (double)vec2.first, maxY / (double)vec2.second ) );

    double mat[2][2] = {
      { vec1.first, vec1.second },
      { vec2.first, vec2.second }
    };

    double det = mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0];
    if( sgn(det) == 0 )
      return ret;

    // luam adjuncta
    std::swap( mat[0][0], mat[1][1] );
    std::swap( mat[1][0], mat[0][1] );
    mat[1][0] *= -1;
    mat[0][1] *= -1;

    // impartim la det ca sa gasim inversa
    mat[0][0] /= det;
    mat[0][1] /= det;
    mat[1][0] /= det;
    mat[1][1] /= det;

    double a1 = mat[0][0] * maxX + mat[0][1] * maxY;
    double a2 = mat[1][0] * maxX + mat[1][1] * maxY;

    //assert( sgn(vec1.first * a1 + vec2.first * a2 - maxX) == 0 );
    //assert( sgn(vec1.second * a1 + vec2.second * a2 - maxY) == 0 );

    if( a1 < 0 || a2 < 0 )
      return ret;

    ret = std::max( ret, a1 + a2 );
    return ret;
  }

  bool by_slope( const std::pair<int, int> &a, const std::pair<int, int> &b ) {
    return a.first * (ll)b.second < b.first * (ll)a.second;
  };

#define cvec const std::vector<int> &

  std::vector<std::pair<int, int>> points0;
  std::vector<std::pair<int, int>> points1;
  std::vector<std::pair<int, int>> points2;

  void init( cvec __A, cvec __B ) {
    A = __A;
    B = __B;
    n = (int)A.size();

    points0.clear(); {
      for( int i = 0; i < n; i++ )
        points0.emplace_back( A[i], B[i] );
      std::sort( points0.begin(), points0.end() );
    }

    points1.clear(); {// yeet useless vectors (for now only (0, inf) subtask)
      std::reverse( points0.begin(), points0.end() );
      for( auto [x, y]: points0 ){
        while( !points1.empty() && points1.back().second >= y )
          points1.pop_back();
        points1.emplace_back( x, y );
      }

      std::reverse( points1.begin(), points1.end() );
      std::reverse( points0.begin(), points0.end() );
    }

    if( false )
      for( int i = 1; i < (int)points1.size(); i++ )
        assert( by_slope( points1[i - 1], points1[i] ) );

    points2.clear(); {// now make it convex (aparently is wasnt before)
      auto convex_mi_bombobclat = [](
        const std::pair<int, int> &a,
        const std::pair<int, int> &b,
        const std::pair<int, int> &c
      ) -> bool {
        ll det = (a.first - b.first) * (ll)(a.second - c.second) - (a.second - b.second) * (ll)(a.first - c.first);
        return det < 0;
      };

      for( auto P : points1 ){
        while( (int)points2.size() >= 2 && convex_mi_bombobclat( points2.end()[-2], points2.end()[-1], P ) > 0 )
          points2.pop_back();
        points2.push_back( P );
      }
    }

    if( false ){
      for( auto [x, y]: points2 )
        printf( "(%d, %d)\n", x, y );
    }
  }

  double query( int maxX, int maxY, int minA, int maxA ) {
    if( true || (minA == 0 && maxA == 1000000000) ) {
      int begin = std::lower_bound( points2.begin(), points2.end(), std::pair<int, int>(minA, 0) ) - points2.begin();
      int end   = std::lower_bound( points2.begin(), points2.end(), std::pair<int, int>(maxA + 1, 0) ) - points2.begin();

      int begin1, end1; {// range1
        begin1 = std::lower_bound( points0.begin(), points0.end(), std::pair<int, int>(minA, 0) ) - points0.begin();

        if( begin < end )
          end1 = std::upper_bound( points0.begin(), points0.end(), points2[begin] ) - points0.begin();
        else
          end1 = std::lower_bound( points0.begin(), points0.end(), std::pair<int, int>(maxA + 1, 0) ) - points0.begin();
      }

      int begin2, end2; {// range2
        if( begin < end )
          begin2 = std::lower_bound( points0.begin(), points0.end(), points2[end - 1] ) - points0.begin();
        else
          begin2 = std::lower_bound( points0.begin(), points0.end(), std::pair<int, int>(minA, 0) ) - points0.begin();

        end2 = std::lower_bound( points0.begin(), points0.end(), std::pair<int, int>(maxA + 1, 0) ) - points0.begin();
      }

      double ret = 0;
      for( int i1 = begin1; i1 < end1; i1++ )
        for( int i2 = begin2; i2 < end2; i2++ )
          ret = std::max( ret, solve2( maxX, maxY, points0[i1], points0[i2] ) );

      // ret = solve2( maxX, maxY, points[idx - 1], points[idx] );

      // only (0, inf) subtask now...
      return ret;
    }

    std::vector<std::pair<int, int>> variante;
    for( int i = 0; i < n; i++ )
      if( minA <= A[i] && A[i] <= maxA )
        variante.emplace_back( A[i], B[i] );

    double ret = 0;
    for( int i = 0; i < (int)variante.size(); i++ )
      for( int j = i; j < (int)variante.size(); j++ )
        ret = std::max( ret, solve2( maxX, maxY, variante[i], variante[j] ) );

    return ret;
  }
}

void init(const std::vector<int>& A, const std::vector<int>& B) { __Sol__::init( A, B ); }
double friendDrink(const int Ma, const int Mb, const int La, const int Ra) { return __Sol__::query( Ma, Mb, La, Ra ); }
