#include "soft_drinks.h"

#include <vector>
#include <algorithm>

#include <stdio.h>
#include <assert.h>

namespace __Sol__ {
  using ll = long long;
  constexpr double EPS = 1e-8;

  int sgn( double x ) {
    return (x > EPS) - (x < -EPS);
  }

  int n;
  std::vector<int> A, B;

  double solve2( int maxX, int maxY, std::pair<int, int> vec1, std::pair<int, int> vec2 ) {
    // either one -- hits boundary
    // or two -- gets both

    double ret = 0;
    // division by 0 should not matter that much
    ret = std::max( ret, std::min( maxX / (double)vec1.first, maxY / (double)vec1.second ) );
    ret = std::max( ret, std::min( maxX / (double)vec2.first, maxY / (double)vec2.second ) );

    double mat[2][2] = {
      { vec1.first, vec1.second },
      { vec2.first, vec2.second }
    };

    double det = mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0];
    if( sgn(det) == 0 )
      return ret;

    // luam adjuncta
    std::swap( mat[0][0], mat[1][1] );
    std::swap( mat[1][0], mat[0][1] );
    mat[1][0] *= -1;
    mat[0][1] *= -1;

    // impartim la det ca sa gasim inversa
    mat[0][0] /= det;
    mat[0][1] /= det;
    mat[1][0] /= det;
    mat[1][1] /= det;

    double a1 = mat[0][0] * maxX + mat[0][1] * maxY;
    double a2 = mat[1][0] * maxX + mat[1][1] * maxY;

    //assert( sgn(vec1.first * a1 + vec2.first * a2 - maxX) == 0 );
    //assert( sgn(vec1.second * a1 + vec2.second * a2 - maxY) == 0 );

    if( a1 < 0 || a2 < 0 )
      return ret;

    ret = std::max( ret, a1 + a2 );
    return ret;
  }

  bool by_slope( const std::pair<int, int> &a, const std::pair<int, int> &b ) {
    return a.first * (ll)b.second < b.first * (ll)a.second;
  };

  bool convex_mi_bombobclat(
    const std::pair<int, int> &a,
    const std::pair<int, int> &b,
    const std::pair<int, int> &c
  ) {
    ll det = (a.first - b.first) * (ll)(a.second - c.second) - (a.second - b.second) * (ll)(a.first - c.first);
    return det < 0;
  };

#define cvec const std::vector<int> &
  std::vector<std::pair<int, int>> points;
  void init( cvec __A, cvec __B ) {
    A = __A;
    B = __B;
    n = (int)A.size();

    points.clear();
    for( int i = 0; i < n; i++ )
      points.emplace_back( A[i], B[i] );

    std::sort( points.begin(), points.end() );

    // return;

    if( false ){// yeet useless vectors (for now only (0, inf) subtask)
      std::vector<std::pair<int, int>> good;
      std::sort( points.begin(), points.end(), std::greater<std::pair<int, int>>() );
      for( auto [x, y]: points ){
        while( !good.empty() && good.back().second >= y )
          good.pop_back();
        good.emplace_back( x, y );
      }

      std::reverse( good.begin(), good.end() );
      points = good;
    }

    if( false )
      for( int i = 1; i < (int)points.size(); i++ )
        assert( by_slope( points[i - 1], points[i] ) );

    if( true ){// now make it convex (aparently is wasnt before)
      std::vector<std::pair<int, int>> good;

      for( auto P : points ){
        while( (int)good.size() >= 2 && convex_mi_bombobclat( good.end()[-2], good.end()[-1], P ) > 0 )
          good.pop_back();
        good.push_back( P );
      }

      points = good;
    }

    if( false ){
      for( auto [x, y]: points )
        printf( "(%d, %d)\n", x, y );
    }
  }

  double query( int maxX, int maxY, int minA, int maxA ) {
    if( (minA == 0 && maxA == 1000000000) ) {
      int n = (int)points.size();
      int idx = std::lower_bound(
        points.begin(), points.end(),
        std::pair<int, int>(maxX, maxY),
        by_slope
      ) - points.begin();

      double ret = 0;
      if( idx == 0 )
        return std::min( maxX / (double)points[0].first, maxY / (double)points[0].second );
      if( idx == n )
        return std::min( maxX / (double)points[n - 1].first, maxY / (double)points[n - 1].second );

      ret = solve2( maxX, maxY, points[idx - 1], points[idx] );

      // only (0, inf) subtask now...
      return ret;
    }

    std::vector<std::pair<int, int>> variante;
    for( int i = 0; i < n; i++ )
      if( minA <= A[i] && A[i] <= maxA )
        variante.emplace_back( A[i], B[i] );

    double ret = 0;
    for( int i = 0; i < (int)variante.size(); i++ )
      for( int j = i; j < (int)variante.size(); j++ )
        ret = std::max( ret, solve2( maxX, maxY, variante[i], variante[j] ) );

    return ret;
  }

  /*double query( int maxX, int maxY, int minA, int maxA ) {
    double ret = 0;

    { // left side, all smaller thatn maxA
      std::vector<std::pair<int, int>> stack;
      for( int i = 0; i < (int)points.size() && points[i].first <= maxA; i++ ){
        while( (int)stack.size() >= 2  )
      }
    }
    return ret;
  }*/
}

void init(const std::vector<int>& A, const std::vector<int>& B) { __Sol__::init( A, B ); }
double friendDrink(const int Ma, const int Mb, const int La, const int Ra) { return __Sol__::query( Ma, Mb, La, Ra ); }