#include "soft_drinks.h"

#include <vector>
#include <algorithm>
#include <random>

#include <stdio.h>
#include <assert.h>

namespace __Sol__ {
  /*struct Point {
    int x, y;
    Point( int x, int y ): x(x), y(y) {}
    bool operator < ( const Point& that ) const {
      if( x != that.x )
        return x < that.x;
      return y < that.y;
    }
  };*/

  using ll = long long;
  using Point = std::pair<int, int>;

  constexpr double EPS = 1e-8;
  int sgn( double x ) { return (x > EPS) - (x < -EPS); }


  bool convex_mi_bombobclat( const Point &a, const Point &b, const Point &c ) {
    ll det = (a.first - b.first) * (ll)(a.second - c.second) - (a.second - b.second) * (ll)(a.first - c.first);
    return det <= 0;
  };

  namespace TreapImp {
    std::mt19937 rng(42);
    struct Treap {
      int left, right;
      Point P;
      ll pri;

      Treap( const Point &P = Point() ): left(0), right(0), P(P) { pri = rng(); }
    };

    std::vector<Treap> t(1);

    int new_node() {
      int ret = (int)t.size();
      t.emplace_back();
      return ret;
    }

    int new_node( const Point &P ) {
      int ret = (int)t.size();
      t.emplace_back( P );
      return ret;
    }

    int copy( int node ) {
      int ret = new_node();
      t[ret] = t[node];
      return ret;
    }

    int join( int left, int right ){
      if( !left ) return right;
      if( !right ) return left;

      if( t[left].pri > t[right].pri ){
        int root = copy( left );
        t[root].right = join( t[root].right, right );
        return root;
      }else{
        int root = copy( right );
        t[root].left = join( left, t[root].left );
        return root;
      }
    }

    template<class CB> std::pair<int, int> split( int root, CB &&isleft ){
      if( !root ) return { 0, 0 };

      if( isleft( root ) ){
        int newroot = copy( root );
        auto [rleft, right] = split( t[newroot].left, isleft );
        t[newroot].left = rleft;
        return { newroot, right };
      }else{
        int newroot = copy( root );
        auto [left, rright] = split( t[newroot].right, isleft );
        t[newroot].right = rright;
        return { left, newroot };
      }
    }

    std::pair<int, int> split_val( int root, Point P ) {
      return split( root, [&]( int root ) -> bool {
        return t[root].P < P;
      } );
    }

    // this joins two convex hulls
    int join_hulls( int left, int right ) {
      return 0;
    }
  }

  struct Node {
    std::vector<Point> points;
    Node(): points() {}
    Node( Point P ): points(1, P) {}
    Node operator + ( const Node& that ) const {
      Node ret(*this);
      for( Point P: that.points ){
        if( (int)ret.points.size() >= 1 && P.second > ret.points.back().second )
          continue;

        while( (int)ret.points.size() >= 2 && convex_mi_bombobclat( ret.points.end()[-2], ret.points.end()[-1], P ) )
          ret.points.pop_back();
        ret.points.push_back( P );
      }

      return ret;
    }
  };

  struct AINT {
    std::vector<Node> aint;
    int aintn;
    void init( int n ) {
      aintn = 1;
      while( aintn < n )
        aintn <<= 1;

      aint.assign( 2 * aintn, Node() );
    }

    Node& operator []( int idx ) { return aint[aintn - 1 + idx]; }
    void build() {
      for( int i = aintn - 1; i--; )
        aint[i] = aint[i * 2 + 1] + aint[i * 2 + 2];
    }

    Node query( int st, int dr ) {
      Node ret_left, ret_right;

      st += aintn - 1;
      dr += aintn - 1;

      if( st > dr )
        return ret_left;

      while( st < dr ){
        if( !(st & 1) )
          ret_left = ret_left + aint[st++];
        if( dr & 1 )
          ret_right = aint[dr--] + ret_right;

        st = (st - 1) >> 1;
        dr = (dr - 1) >> 1;
      }

      if( st == dr )
        ret_left = ret_left + aint[st];

      return ret_left + ret_right;
    }
  };

  double solve2( int maxX, int maxY, std::pair<int, int> vec1, std::pair<int, int> vec2 ) {
    // either one -- hits boundary
    // or two -- gets both

    double ret = 0;
    // division by 0 should not matter that much
    ret = std::max( ret, std::min( maxX / (double)vec1.first, maxY / (double)vec1.second ) );
    ret = std::max( ret, std::min( maxX / (double)vec2.first, maxY / (double)vec2.second ) );

    double mat[2][2] = {
      { vec1.first, vec1.second },
      { vec2.first, vec2.second }
    };

    double det = mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0];
    if( sgn(det) == 0 )
      return ret;

    // luam adjuncta
    std::swap( mat[0][0], mat[1][1] );
    std::swap( mat[1][0], mat[0][1] );
    mat[1][0] *= -1;
    mat[0][1] *= -1;

    // impartim la det ca sa gasim inversa
    mat[0][0] /= det;
    mat[0][1] /= det;
    mat[1][0] /= det;
    mat[1][1] /= det;

    double a1 = mat[0][0] * maxX + mat[0][1] * maxY;
    double a2 = mat[1][0] * maxX + mat[1][1] * maxY;

    //assert( sgn(vec1.first * a1 + vec2.first * a2 - maxX) == 0 );
    //assert( sgn(vec1.second * a1 + vec2.second * a2 - maxY) == 0 );

    if( a1 < 0 || a2 < 0 )
      return ret;

    ret = std::max( ret, a1 + a2 );
    return ret;
  }

  bool by_slope( const std::pair<int, int> &a, const std::pair<int, int> &b ) {
    return a.first * (ll)b.second < b.first * (ll)a.second;
  };

#define cvec const std::vector<int> &

  std::vector<std::pair<int, int>> points;
  AINT zile;
  void init( cvec A, cvec B ) {
    int n = (int)A.size();
    for( int i = 0; i < n; i++ )
      points.emplace_back( A[i], B[i] );

    std::sort( points.begin(), points.end() );

    zile.init( n );
    for( int i = 0; i < n; i++ )
      zile[i] = points[i];
    zile.build();

    // if( false ){// yeet useless vectors (for now only (0, inf) subtask)
    //   std::vector<std::pair<int, int>> good;
    //   std::sort( points.begin(), points.end(), std::greater<std::pair<int, int>>() );
    //   for( auto [x, y]: points ){
    //     while( !good.empty() && good.back().second >= y )
    //       good.pop_back();
    //     good.emplace_back( x, y );
    //   }

    //   std::reverse( good.begin(), good.end() );
    //   points = good;
    // }

    // if( false )
    //   for( int i = 1; i < (int)points.size(); i++ )
    //     assert( by_slope( points[i - 1], points[i] ) );

    // if( true ){// now make it convex (aparently is wasnt before)
    //   std::vector<std::pair<int, int>> good;

    //   for( auto P : points ){
    //     while( (int)good.size() >= 2 && convex_mi_bombobclat( good.end()[-2], good.end()[-1], P ) > 0 )
    //       good.pop_back();
    //     good.push_back( P );
    //   }

    //   points = good;
    // }

    // if( false ){
    //   for( auto [x, y]: points )
    //     printf( "(%d, %d)\n", x, y );
    // }
  }

  double query( int maxX, int maxY, int minA, int maxA ) {
    Node variante = zile.query(
      std::lower_bound( points.begin(), points.end(), std::pair<int, int>(minA,     0) ) - points.begin(),
      std::lower_bound( points.begin(), points.end(), std::pair<int, int>(maxA + 1, 0) ) - points.begin() - 1
    );

    // double ret = 0;
    const std::vector<Point> &vec = variante.points;
    if( vec.empty() )
      return 0;

    // printf( "vec:" );
    // for( auto [x, y]: vec )
    //   printf( " (%d, %d)", x, y );
    // printf( "\n" );

    int n = (int)vec.size();
    int idx = std::lower_bound(
      vec.begin(), vec.end(),
      Point(maxX, maxY),
      by_slope
    ) - vec.begin();

    if( idx == 0 || idx == n )
      return solve2( maxX, maxY, vec.front(), vec.back() );

    return solve2( maxX, maxY, vec[std::max( 0, idx - 1 )], vec[std::min( n - 1, idx )] );
  }
}

void init(const std::vector<int>& A, const std::vector<int>& B) { __Sol__::init( A, B ); }
double friendDrink(const int Ma, const int Mb, const int La, const int Ra) { return __Sol__::query( Ma, Mb, La, Ra ); }