#include "soft_drinks.h"
#include <iostream>
#include <algorithm>
#include <iomanip>
#include <cassert>

#define endl '\n'
#define x first
#define y second

using namespace std;

typedef long long ll;

const double INF = 1000000000.0 + 10.0;
const double EPS = 0.000001;

vector < pair < double, double > > drinks;
int n;

void init(const std::vector<int>& A, const std::vector<int>& B)
{
    n = A.size();
    drinks.resize(n);

    for(int i = 0; i < n; i++)
    {
        drinks[i] = {(double)(A[i]), (double)(B[i])};
    }

    //cout << setprecision(7) << EPS << endl;

    sort(drinks.begin(), drinks.end());
}

double check(double L0, double Ma, double Mb)
{
    double sugar = L0 * drinks[0].x;
    double acid = L0 * drinks[0].y;

    double left_sugar = Ma - sugar;
    double left_acid = Mb - acid;

    double byA = (drinks[1].x == 0 ? INF : (double)((double)(left_sugar) / drinks[1].x));
    double byB = (drinks[1].y == 0 ? INF : (double)((double)(left_acid) / drinks[1].y));

    return L0 + min(byA, byB);
}

double friendDrink(const int Ma, const int Mb, const int La, const int Ra)
{
    if(n > 2)
        return -1;

    if(n == 1)
    {
        bool in0 = (La <= drinks[0].x && drinks[0].x <= Ra);

        if(!in0)
            return 0;
        
        double byA = (drinks[0].x == 0 ? INF : (double)((double)(Ma) / drinks[0].x));
        double byB = (drinks[0].y == 0 ? INF : (double)((double)(Mb) / drinks[0].y));

        return min(byA, byB);
    }   

    bool in0 = (La <= drinks[0].x && drinks[0].x <= Ra);
    bool in1 = (La <= drinks[1].x && drinks[1].x <= Ra);

    if(!in0 && !in1)
    {
        //cout << "None" << endl;
        return 0;
    }
    else if(!in0)
    {
        //cout << "Only 1" << endl; 
        double byA = (drinks[1].x == 0 ? INF : (double)((double)(Ma) / drinks[1].x));
        double byB = (drinks[1].y == 0 ? INF : (double)((double)(Mb) / drinks[1].y));

        return min(byA, byB);
    }
    else if(!in1)
    {
        //cout << "Only 0" << endl;
        double byA = (drinks[0].x == 0 ? INF : (double)((double)(Ma) / drinks[0].x));
        double byB = (drinks[0].y == 0 ? INF : (double)((double)(Mb) / drinks[0].y));

        return min(byA, byB);
    }

    //cout << "Both" << endl;

    assert(in0 && in1);

    double left = 0;
    double right = INF + EPS;
    double mid;

    while(right - left > EPS)
    {
        mid = (left + right) / 2.0;

        if(check(mid, (double)(Ma), (double)(Mb)) <= check(mid + EPS, (double)(Ma), (double)(Mb)))
            left = mid;
        else
            right = mid;
    }

    double first = check(left, (double)(Ma), (double)(Mb));
    //cout << left << endl;

    return first;
    /*
    double nofirst = check((double)(0.0), (double)(Ma), (double)(Mb));

    swap(drinks[0], drinks[1]);

    left = 0;
    right = INF + EPS;

    while(right - left > EPS)
    {
        mid = (left + right) / 2.0;

        if(check(mid, (double)(Ma), (double)(Mb)) <= check(mid + EPS, (double)(Ma), (double)(Mb)))
            left = mid;
        else
            right = mid;
    }

    double second = check(left, (double)(Ma), (double)(Mb));
    double nosecond = check((double)(0.0), (double)(Ma), (double)(Mb));

    swap(drinks[0], drinks[1]);


    //cout << "Possible : " << endl;
    //cout << first << " - " << nofirst << endl;
    //cout << second << " - " << nosecond << endl;


    return max({first, second, nofirst, nosecond});

    */
}
