#include <bits/stdc++.h>
#define endl '\n'

using namespace std;

const int MAXN = 2e5+50;
class Tree
{
private:
    struct Edge
    {
        int index;
        int64_t weight;

        Edge(){}
        Edge(int index, int64_t weight)
            :index(index), weight(weight){}
    };

    int N;
    int parent[MAXN];
    vector <Edge> graph[MAXN];
    vector <Edge> children[MAXN];
    int64_t dp1[MAXN], dp2[MAXN];
    bool used[MAXN];

    void MEMSET_USED()
    {
        memset(used, false, sizeof(used));
    }
    void CalculateDP(int currNode)
    {
        used[currNode] = true;
        if (children[currNode].size() == 0)
        {
            dp1[currNode] = dp2[currNode] = 0;
            return;
        }

        for (auto [nextNode, weight]:children[currNode])
        {
            if (used[nextNode])
                continue;

            CalculateDP(nextNode);
        }

        if (children[currNode].size() > 1)
        {
            int max1 = 0, max2 = 0;
            for (int i = 0; i < children[currNode].size(); i++)
            {
                int child = children[currNode][i].index;
                int weight = children[currNode][i].weight;
                if (dp1[child]+weight >= max1)
                {
                    max2 = max1;
                    max1 = dp1[child]+weight;
                }

                else if (dp1[child]+weight > max2)
                {
                    max2 = dp1[child]+weight;
                }
            }

            dp2[currNode] = max1+max2;
        }

        for (int i = 0; i < children[currNode].size(); i++)
        {
            int child = children[currNode][i].index;
            int weight = children[currNode][i].weight;
            dp1[currNode] = max(dp1[currNode], dp1[child]+weight);
        }
    }

    void ConstructTree(int currNode)
    {
        used[currNode] = true;
        for (auto [nextNode, weight]:graph[currNode])
        {
            if (used[nextNode])
                continue;

            children[currNode].emplace_back(nextNode, weight);
            parent[nextNode] = currNode;

            ConstructTree(nextNode);
        }
    }
public:
    Tree(int N)
    {
        this->N = N;
        memset(dp1, false, sizeof(dp1));
        memset(dp2, false, sizeof(dp2));
    }

    void AddEdge(int u, int v, int64_t weight)
    {
        graph[u].emplace_back(v, weight);
        graph[v].emplace_back(u, weight);
    }

    int64_t FindDiameter()
    {
        int currNode = 1;
        parent[currNode] = 0;

        MEMSET_USED();
        ConstructTree(currNode);

        MEMSET_USED();
        CalculateDP(currNode);

        int64_t diameter = 0;
        for (int i = 1; i <= N; i++)
        {
            //cout << "Node #" << i << " - " << dp1[i] << " " << dp2[i] << endl;
            diameter = max(diameter, max(dp1[i], dp2[i]));
        }
        return diameter;
    }
};

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(0); cout.tie(0);

    int N, K;
    cin >> N >> K;

    Tree *tree = new Tree(N);
    for (int i = 0; i < N-1; i++)
    {
        int u, v;
        int64_t weight;
        cin >> u >> v >> weight;

        tree->AddEdge(u, v, weight);
    }

    cout << tree->FindDiameter()-1 << endl;

    return 0;
}
