/*
    Zarad tebe vdigam vurtoleta
*/

#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <random>
#include <chrono>
#include <vector>
#include <stack>
#include <queue>

#include <ext/pb_ds/assoc_container.hpp>
#include <ext/pb_ds/tree_policy.hpp>

template <typename T>
using OSet = __gnu_pbds::tree <T, __gnu_pbds::null_type, std::less <T>, __gnu_pbds::rb_tree_tag, __gnu_pbds::tree_order_statistics_node_update>;

typedef long long llong;
// const llong TIME = std::chrono::steady_clock::now().time_since_epoch().count();
const int MAXN = 100000 + 10;
const int MAXLOG = 17;
const int INF  = 2e9 + 1;
const int DIG_LEVELS = 7;

int n;
int a[MAXN];
int b[MAXN];

struct Sparse
{
    int par[MAXLOG][MAXN];
    void build()
    {
        for (int lg = 1 ; (1 << lg) <= n ; ++lg)
        {
            for (int i = 1 ; i <= n ; ++i)
            {
                par[lg][i] = par[lg - 1][par[lg - 1][i]];
            }
        }
    }
};

Sparse sparse[DIG_LEVELS];

double checkTwo(double a1, double b1, double a2, double b2, double mA, double mB)
{
    if (a1 <= a2 && b1 <= b2)
    {
        return 0;
    }

    if (a2 <= a1 && b2 <= b1)
    {
        return 0;
    }

    if (a1 < a2)
    {
        std::swap(a1, a2);
        std::swap(b1, b2);
    }

    double s = (mA * (b2 - b1) + mB * (a1 - a2)) / (a1 * b2 - a2 * b1);
    if (mA < a2 * s) return 0;
    if (mB < b1 * s) return 0;
    // if ((\1 * s - mA) / (a1 - a2) > s) return 0;
    return s;
}

double checkOne(double a, double b, double mA, double mB)
{
    return std::min(mA / a, mB / b);
}

struct Point
{
    double x, y;
};

double size(Point a, Point b, Point c)
{
    return a.x * b.y + a.y * c.x + b.x * c.y - b.y * c.x - a.x * c.y - a.y * b.x;
}

Point p[MAXN];
void rec(int l, int r, int lvl)
{
    // std::cout << "rec: " << l << ' ' << r << " " << lvl << '\n' << std::flush;
    if (lvl >= DIG_LEVELS)
    {
        return;
    }

    int mid = l + r >> 1;
    if (l != r)
    {
            rec(l, mid, lvl + 1);
        rec(mid + 1, r, lvl + 1);
    }

    std::vector <int> hull;
    for (int i = mid + 1 ; i <= r ; ++i)
    {
        if (hull.empty() || b[i] < p[hull.back()].y) 
        {
            while (hull.size() >= 2 && size(p[hull[hull.size() - 2]], p[hull[hull.size() - 1]], p[i]) <= 0)
            {
                hull.pop_back();
            }
            
            hull.push_back(i);
        }
        
        sparse[lvl].par[0][i] = (hull.empty() ? 0 : hull.back());
    }

    hull.clear();
    for (int i = mid ; i >= l ; --i)
    {
        sparse[lvl].par[0][i] = (hull.empty() ? 0 : hull.back());
        if (hull.size() >= 2 && size(p[i], p[hull[hull.size() - 1]], p[hull[hull.size() - 2]]) > 0)
        {
            hull.push_back(i);
        }        
    }
}


void init(const std::vector <int> &A, const std::vector <int> &B)
{
    n = A.size();
    std::vector <std::pair <int,int>> v;
    for (int i = 0 ; i < n ; ++i) v.push_back({A[i], B[i]});

    std::sort(v.begin(), v.end());
    for (int i = 1 ; i <= n ; ++i)
    {
        a[i] = v[i - 1].first;
        b[i] = v[i - 1].second;
        p[i] = {a[i], b[i]};
    }

    rec(1, n, 0);
    for (int i = 0 ; i < DIG_LEVELS ; ++i)
    {
        sparse[i].build();
    }
}

double friendDrink(int mA, int mB, int lA, int rA)
{
    {
        int l = 0, r = n + 1, mid;
        while (l < r - 1)
        {
            mid = (l + r) >> 1;
            if (p[mid].x < lA) l = mid;
            else r = mid;
        }

        lA = r;
    }

    {
        int l = 0, r = n + 1, mid;
        while (l < r - 1)
        {
            mid = (l + r) >> 1;
            if (p[mid].x <= rA) l = mid;
            else r = mid;
        }

        rA = l;
    }

    if (lA > rA)
    {
        return 0;
    }

    // std::cout << "here: " << lA << ' ' << rA << '\n' << std::flush;
    int ll = 1, rr = n, lvl = 0;

    while (lvl < DIG_LEVELS)
    {
        int mid = ll + rr >> 1;
        if (lA <= mid && mid + 1 <= rA)
        {
            break;
        }

        if (lA <= mid)
        {
            rr = mid;
        } else
        {
            ll = mid + 1;
        }

        lvl++;
    }

    if (lvl == DIG_LEVELS || rA - lA <= 1000)
    {
        double ans = 0.0;
        std::vector <int> hull;
        for (int i = lA ; i <= rA ; ++i)
        {
            ans = std::max(ans, checkOne(a[i], b[i], mA, mB));
            if (hull.empty() || b[i] < p[hull.back()].y) 
            {
                while (hull.size() >= 2 && size(p[hull[hull.size() - 2]], p[hull[hull.size() - 1]], p[i]) <= 0)
                {
                    hull.pop_back();
                }
                
                if (hull.size()) ans = std::max(ans, checkTwo(a[i], b[i], a[hull.back()], b[hull.back()], mA, mB));
                hull.push_back(i);
            }
        }

        return ans;
    }

    // std::cout << "not dig levels\n";
    double ans = 0.0;
    int start = sparse[lvl].par[0][lA];
    if (start != 0)
    {
        ans = std::max(ans, checkTwo(a[lA], b[lA], a[start], b[start], mA, mB));
    }

    int endL = lA;
    for (int lg = MAXLOG - 1 ; lg >= 0 ; --lg)
    {
        if (sparse[lvl].par[lg][endL] != 0)
        {
            endL = sparse[lvl].par[lg][endL];
        }
    }

    if (mA * p[endL].y <= mB * p[endL].x)
    {
        // std::cout << "first\n";
        int split = sparse[lvl].par[0][lA];
        for (int lg = MAXLOG - 1 ; lg >= 0 ; --lg)
        {
            if (sparse[lvl].par[lg][split] != 0 && mA * p[sparse[lvl].par[lg][split]].y > mB * p[sparse[lvl].par[lg][split]].x)
            {
                split = sparse[lvl].par[lg][split];
            }
        }

        if (split != 0) ans = std::max(ans, checkOne(a[split], b[split], mA, mB));
        if (sparse[lvl].par[0][split] != 0)
        {
            ans = std::max(ans, checkOne(a[sparse[lvl].par[0][split]], b[sparse[lvl].par[0][split]], mA, mB));
            ans = std::max(ans, checkTwo(a[split], b[split], a[sparse[lvl].par[0][split]], b[sparse[lvl].par[0][split]], mA, mB));
        }

        return ans;
    }

    start = sparse[lvl].par[0][rA];
    if (start != 0)
    {
        ans = std::max(ans, checkTwo(a[rA], b[rA], a[start], b[start], mA, mB));
    }

    int endR = rA;
    for (int lg = MAXLOG - 1 ; lg >= 0 ; --lg)
    {
        if (sparse[lvl].par[lg][endR] != 0)
        {
            endR = sparse[lvl].par[lg][endR];
        }
    }

    if (mA * p[endR].y >= mB * p[endR].x)
    {
        int split = sparse[lvl].par[0][rA];
        for (int lg = MAXLOG - 1 ; lg >= 0 ; --lg)
        {
            if (sparse[lvl].par[lg][split] != 0 && mA * p[sparse[lvl].par[lg][split]].y < mB * p[sparse[lvl].par[lg][split]].x)
            {
                split = sparse[lvl].par[lg][split];
            }
        }

        // std::cout << "split: " << split << '\n';
        if (split != 0) ans = std::max(ans, checkOne(a[split], b[split], mA, mB));
        if (sparse[lvl].par[0][split] != 0)
        {
            ans = std::max(ans, checkOne(a[sparse[lvl].par[0][split]], b[sparse[lvl].par[0][split]], mA, mB));
            ans = std::max(ans, checkTwo(a[split], b[split], a[sparse[lvl].par[0][split]], b[sparse[lvl].par[0][split]], mA, mB));
        }

        return ans;
    }

    // std::cout << "third\n";
    // int first = lA;
    // if (sparse[lvl].par[0][lA] == 0)
    // {
    //     ans = std::max(ans, checkTwo(a[lA], b[lA], a[endR], b[endR], mA, mB));
    //     return ans;
    // }

    // first = sparse[lvl].par[0][first];
    // for (int lg = MAXLOG - 1 ; lg >= 0 ; --lg)
    // {
    //     int curr = sparse[lvl].par[lg][first];
    //     int hisParent = sparse[lvl].par[0][first];
    //     if (hisParent != 0 && size(p[curr], p[hisParent], p[endR]) > 0)
    //     {
    //         first = curr;
    //     }
    // }

    // ans = std::max(ans, checkTwo(a[first], b[first], a[endR], b[endR], mA, mB));
    ans = std::max(ans, checkOne(a[endR], b[endR], mA, mB));
    // if (sparse[lvl].par[0][first] != 0)
    // {
    //     ans = std::max(ans, checkTwo(a[sparse[lvl].par[0][first]], b[sparse[lvl].par[0][first]], a[endR], b[endR], mA, mB));
    //     ans = std::max(ans, checkOne(a[sparse[lvl].par[0][first]], b[sparse[lvl].par[0][first]], mA, mB));
    // }

    for (int x = mid ; x >= std::max(lA, mid - 1000) ; ++x)
    {
        
        ans = std::max(ans, checkTwo(a[x], b[x], a[endR], b[endR], mA, mB));
    }

    return ans;
}