/*
    Zarad tebe vdigam vurtoleta
*/

#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <random>
#include <chrono>
#include <vector>
#include <stack>
#include <queue>

#include <ext/pb_ds/assoc_container.hpp>
#include <ext/pb_ds/tree_policy.hpp>

template <typename T>
using OSet = __gnu_pbds::tree <T, __gnu_pbds::null_type, std::less <T>, __gnu_pbds::rb_tree_tag, __gnu_pbds::tree_order_statistics_node_update>;

typedef long long llong;
// const llong TIME = std::chrono::steady_clock::now().time_since_epoch().count();
const int MAXN = 2000 + 10;
const int MAXLOG = 20;
const int INF  = 2e9;

int n, k;
struct SegmentTreeNodes
{
    struct Node
    {
        llong max;
        llong lazy;
        int maxIdx;

        Node()
        {
            max = lazy = maxIdx = 0;
        }

        friend Node operator + (const Node &left, const Node &right)
        {
            Node res;
            res.max = std::max(left.max, right.max);
            if (res.max == left.max)
            {
                res.maxIdx = left.maxIdx;
            } else
            {
                res.maxIdx = right.maxIdx;
            }

            return res;
        }
    };

    Node tree[4*MAXN];
    void build(int l, int r, int node)
    {
        tree[node] = Node();
        if (l == r)
        {
            tree[node].maxIdx = l;
            return;
        }

        int mid = l + r >> 1;
        build(l, mid, 2*node);
        build(mid + 1, r, 2*node + 1);
    }

    void push(int node, int l, int r)
    {
        if (tree[node].lazy == 0)
        {
            return;
        }

        tree[node].max += tree[node].lazy;
        // tree[node].max2 += tree[node].lazy;
        if (l != r)
        {
            tree[2*node].lazy += tree[node].lazy;
            tree[2*node + 1].lazy += tree[node].lazy;
        }

        tree[node].lazy = 0;
    }

    void update(int l, int r, int node, int queryL, int queryR, int queryVal)
    {
        push(node, l, r);
        if (queryR < l || r < queryL)
        {
            return;
        }

        if (queryL <= l && r <= queryR)
        {
            tree[node].lazy += queryVal;
            push(node, l, r);
            return;
        }

        int mid = l + r >> 1;
        update(l, mid, 2*node, queryL, queryR, queryVal);
        update(mid + 1, r, 2*node + 1, queryL, queryR, queryVal);
        tree[node] = tree[2*node] + tree[2*node + 1];
    }

    Node query(int l, int r, int node, int queryL, int queryR)
    {
        push(node, l, r);
        if (queryL <= l && r <= queryR)
        {
            return tree[node];
        }

        int mid = l + r >> 1;
        Node res;

        if (queryL <= mid) res = res + query(l, mid, 2*node, queryL, queryR);
        if (mid + 1 <= queryR) res = res + query(mid + 1, r, 2*node + 1, queryL, queryR);
        return res;
    }

    void build()
    {
        build(1, n, 1);
    }

    void update(int l, int r, int val)
    {
        update(1, n, 1, l, r, val);
    }

    Node query(int l, int r)
    {
        return query(1, n, 1, l, r);
    }
};

llong remaining[MAXN];
bool removed[MAXN];

struct SegmentTreeEdges
{
    struct Edge
    {
        int dep;
        int l, r;
        int idx;

        friend bool operator < (const Edge &a, const Edge &b)
        {
            return a.dep > b.dep;
        }
    };

    std::vector <Edge> tree[4*MAXN];
    void build(int l, int r, int node)
    {
        while (tree[node].size())
        {
            tree[node].pop_back();
        }
        
        tree[node].push_back({INF, 0, 0, 0});
        if (l == r)
        {
            return;
        }

        int mid = l + r >> 1;
        build(l, mid, 2*node);
        build(mid + 1, r, 2*node + 1);
    }

    void update(int l, int r, int node, int queryL, int queryR, Edge toAdd)
    {
        if (queryL <= l && r <= queryR)
        {
            tree[node].push_back(toAdd);
            return;
        }

        int mid = l + r >> 1;
        if (queryL <= mid) update(l, mid, 2*node, queryL, queryR, toAdd);
        if (mid + 1 <= queryR) update(mid + 1, r, 2*node + 1, queryL, queryR, toAdd);
    }

    Edge query(int l, int r, int node, int queryPos)
    {
        while (removed[tree[node].back().idx])
        {
            tree[node].pop_back();
        }

        Edge top = tree[node].back();
        if (l == r)
        {
            return top;
        }

        int mid = l + r >> 1;
        if (queryPos <= mid) top = std::max(top, query(l, mid, 2*node, queryPos));
        else top = std::max(top, query(mid + 1, r, 2*node + 1, queryPos));
        return top;
    }

    void build()
    {
        build(1, n, 1);
    }

    void update(int l, int r, Edge val)
    {
        update(1, n, 1, l, r, val);
    }

    Edge query(int idx)
    {
        return query(1, n, 1, idx);
    }
};

SegmentTreeNodes treeNodes;
SegmentTreeEdges treeEdges;
struct GraphEdge
{
    int to;
    int cost;
    int idx;
};

int in[MAXN];
int out[MAXN];
int dep[MAXN];
int parent[MAXN];
std::vector <int> tour;
std::vector <GraphEdge> g[MAXN];
llong cost;
llong t;

void initDFS(int node, int par)
{
    in[node] = tour.size();
    tour.push_back(node);
    parent[node] = par;
    dep[node] = dep[par] + 1;
    for (auto &u : g[node])
    {
        if (u.to == parent[node])
        {
            std::swap(u, g[node].back());
            g[node].pop_back();
            break;
        }
    }

    for (const auto &[u, cost, idx] : g[node])
    {
        initDFS(u, node);
    }

    out[node] = tour.size() - 1;
}

struct Range
{
    llong value;
    int l, r;
    int maxChildIdx;
    friend bool operator < (const Range &a, const Range &b)
    {
        return a.value < b.value;
    }
};

void makeLessThan(int idx, llong value, llong target)
{
    while (value > target)
    {
        // std::cout << "make less: " << tour[idx] << ' ' << value << ' ' << target << '\n' << std::flush;
        SegmentTreeEdges::Edge curr = treeEdges.query(idx);
        assert(curr.idx != 0);
        llong toRemove = std::min(remaining[curr.idx], value - target);
        // std::cout << "call update: " << curr.l << ' ' << curr.r << ' ' << toRemove << '\n' << std::flush;
        treeNodes.update(curr.l, curr.r, -toRemove);
        remaining[curr.idx] -= toRemove;
        value -= toRemove;
        cost += toRemove;

        if (remaining[curr.idx] == 0)
        {
            removed[curr.idx] = true;
        }
    }
}

void dfs(int node)
{
    // std::cout << "dfs: " << node << '\n' << std::flush;
    for (const auto &[u, cost, idx] : g[node])
    {
        dfs(u);
    }

    std::priority_queue <Range> pq;
    for (const auto &[u, cost, idx] : g[node])
    {
        // std::cout << "call update: " << in[u] << ' ' << out[u] << '\n' << std::flush;
        treeEdges.update(in[u], out[u], {dep[u], in[u], out[u], idx});
        treeNodes.update(in[u], out[u], cost);
        remaining[idx] = cost;

        SegmentTreeNodes::Node res = treeNodes.query(in[u], out[u]);
        pq.push({res.max, in[u], out[u], res.maxIdx});
    }

    // std::cout << "updated: " << node << ' ' << pq.size() << '\n' << std::flush;
    if (pq.empty())
    {
        return;
    }

    if (pq.size() == 1)
    {
        while (pq.top().value > t)
        {
            auto [value, l, r, idx] = pq.top();
            pq.pop();
            makeLessThan(idx, value, t);
            SegmentTreeNodes::Node res = treeNodes.query(l, r);
            pq.push({res.max, l, r, res.maxIdx});
        }

        return;
    }

    llong atLeast = (t / 2) + (t % 2);
    while (pq.top().value > atLeast)
    {
        auto [value, l, r, idx] = pq.top();
        pq.pop();

        llong target = atLeast;
        if (pq.top().value + atLeast < t)
        {
            target = t - pq.top().value;
        }

        makeLessThan(idx, value, target);
        SegmentTreeNodes::Node res = treeNodes.query(l, r);
        pq.push({res.max, l, r, res.maxIdx});
    }

    while (true)
    {
        auto [value, l, r, idx] = pq.top();
        pq.pop();

        if (value + pq.top().value <= t)
        {
            break;
        }
        
        assert(value + pq.top().value == t + 1);
        llong target = value - 1;
        makeLessThan(idx, value, target);
        SegmentTreeNodes::Node res = treeNodes.query(l, r);
        pq.push({res.max, l, r, res.maxIdx});
    }
}

bool check(llong val)
{
    cost = 0;
    t = val;
    std::fill(removed + 1, removed + n, false);
    std::fill(remaining + 1, remaining + n, 0);
    treeNodes.build();
    treeEdges.build();
    dfs(1);
    // std::cout << "cost: " << cost << '\n';
    return cost <= k;
}

void solve()
{
    tour.push_back(0);
    initDFS(1, 0);
    check(69);
    return;
    // check(6);
    // return;
    llong l = -1, r = INF, mid;
    while (l < r - 1)
    {
        mid = (l + r) / 2;
        if (!check(mid)) l = mid;
        else r = mid;
    }    

    std::cout << r << '\n';
}

void input()
{
    std::cin >> n >> k;
    for (int i = 1 ; i < n ; ++i)
    {
        int u, v, c;
        std::cin >> u >> v >> c;
        g[u].push_back({v, c, i});
        g[v].push_back({u, c, i});
    }
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIOI();
    input();
    solve();

    return 0;
}