/*
    Zarad tebe vdigam vurtoleta
*/

#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <random>
#include <chrono>
#include <vector>
#include <stack>
#include <queue>

#include <ext/pb_ds/assoc_container.hpp>
#include <ext/pb_ds/tree_policy.hpp>

template <typename T>
using OSet = __gnu_pbds::tree <T, __gnu_pbds::null_type, std::less <T>, __gnu_pbds::rb_tree_tag, __gnu_pbds::tree_order_statistics_node_update>;

// typedef long long int;
// const int TIME = std::chrono::steady_clock::now().time_since_epoch().count();
const int MAXN = 200000 + 10;
const int MAXLOG = 20;
const int INF  = 2e9 + 1;

int n, k;
struct SegmentTreeNodes
{
    struct Node
    {
        int max;
        int lazy;
        int maxIdx;

        Node()
        {
            max = lazy = maxIdx = 0;
        }

        friend Node operator + (const Node &left, const Node &right)
        {
            if (left.max > right.max) return left;
            return right;
        }
    };

    Node tree[4*MAXN];
    void build(int l, int r, int node)
    {
        if (l == r)
        {
            tree[node] = Node();
            tree[node].maxIdx = l;
            return;
        }

        int mid = l + r >> 1;
        build(l, mid, 2*node);
        build(mid + 1, r, 2*node + 1);
        tree[node] = tree[2*node] + tree[2*node + 1];
    }

    void push(int node, int l, int r)
    {
        if (tree[node].lazy == 0)
        {
            return;
        }

        tree[node].max += tree[node].lazy;
        if (l != r)
        {
            tree[2*node].lazy += tree[node].lazy;
            tree[2*node + 1].lazy += tree[node].lazy;
        }

        tree[node].lazy = 0;
    }

    void update(int l, int r, int node, int queryL, int queryR, int queryVal)
    {
        push(node, l, r);
        if (queryR < l || r < queryL)
        {
            return;
        }

        if (queryL <= l && r <= queryR)
        {
            tree[node].lazy = queryVal;
            push(node, l, r);
            return;
        }

        int mid = l + r >> 1;
        update(l, mid, 2*node, queryL, queryR, queryVal);
        update(mid + 1, r, 2*node + 1, queryL, queryR, queryVal);
        tree[node] = tree[2*node] + tree[2*node + 1];
    }

    Node query(int l, int r, int node, int queryL, int queryR)
    {
        push(node, l, r);
        if (queryL <= l && r <= queryR)
        {
            return tree[node];
        }

        int mid = l + r >> 1;
        Node res;

        if (queryL <= mid) res = res + query(l, mid, 2*node, queryL, queryR);
        if (mid + 1 <= queryR) res = res + query(mid + 1, r, 2*node + 1, queryL, queryR);
        return res;
    }

    void build()
    {
        build(1, n, 1);
    }

    void update(int l, int r, int val)
    {
        update(1, n, 1, l, r, val);
    }

    Node query(int l, int r)
    {
        return query(1, n, 1, l, r);
    }
};

struct Edge
{
    int dep;
    int l, r;
    int idx;

    friend bool operator < (const Edge &a, const Edge &b)
    {
        return a.dep > b.dep;
    }
};

Edge edges[MAXN];
int remaining[MAXN];
bool removed[MAXN];

struct SegmentTreeEdges
{
    std::vector <int> tree[4*MAXN];
    void build(int l, int r, int node)
    {
        tree[node].clear();
        tree[node].push_back(0);
        if (l == r)
        {
            return;
        }

        int mid = l + r >> 1;
        build(l, mid, 2*node);
        build(mid + 1, r, 2*node + 1);
    }

    void update(int l, int r, int node, int queryL, int queryR, int idx)
    {
        if (queryL <= l && r <= queryR)
        {
            tree[node].push_back(idx);
            return;
        }

        int mid = l + r >> 1;
        if (queryL <= mid) update(l, mid, 2*node, queryL, queryR, idx);
        if (mid + 1 <= queryR) update(mid + 1, r, 2*node + 1, queryL, queryR, idx);
    }

    int query(int l, int r, int node, int queryPos)
    {
        while (removed[tree[node].back()])
        {
            tree[node].pop_back();
        }

        int top = tree[node].back();
        if (l == r)
        {
            return top;
        }

        int mid = l + r >> 1;
        int other;
        if (queryPos <= mid) other = std::max(top, query(l, mid, 2*node, queryPos));
        else other = std::max(top, query(mid + 1, r, 2*node + 1, queryPos));
        if (edges[top].dep > edges[other].dep) top = other;
        return top;
    }

    void build()
    {
        build(1, n, 1);
    }

    void update(int l, int r, int val)
    {
        update(1, n, 1, l, r, val);
    }

    int query(int idx)
    {
        return query(1, n, 1, idx);
    }
};

SegmentTreeNodes treeNodes;
SegmentTreeEdges treeEdges;
struct GraphEdge
{
    int to;
    int cost;
    int idx;
};

int in[MAXN];
int out[MAXN];
int dep[MAXN];
int parent[MAXN];
std::vector <int> tour;
std::vector <GraphEdge> g[MAXN];
int cost;
int t;

void initDFS(int node, int par)
{
    in[node] = tour.size();
    tour.push_back(node);
    parent[node] = par;
    dep[node] = dep[par] + 1;
    for (auto &u : g[node])
    {
        if (u.to == parent[node])
        {
            std::swap(u, g[node].back());
            g[node].pop_back();
            break;
        }
    }

    for (const auto &[u, cost, idx] : g[node])
    {
        initDFS(u, node);
        edges[idx] = {dep[u], in[u], out[u], idx};
    }

    out[node] = tour.size() - 1;
}

struct Range
{
    int value;
    int l, r;
    int maxChildIdx;
    friend bool operator < (const Range &a, const Range &b)
    {
        return a.value < b.value;
    }
};

void makeLessThan(int idx, int value, int target)
{
    while (value > target)
    {
        Edge curr = edges[treeEdges.query(idx)];
        // std::cout << "make less: " << curr.idx << ' ' << remaining[curr.idx] << ' ' << value << ' ' << target << '\n';
        assert(curr.idx != 0);
        assert(remaining[curr.idx] > 0);
        int toRemove = std::min(remaining[curr.idx], value - target);
        treeNodes.update(curr.l, curr.r, -toRemove);
        remaining[curr.idx] -= toRemove;
        value -= toRemove;
        cost += toRemove;

        if (remaining[curr.idx] == 0)
        {
            removed[curr.idx] = true;
        }
    }
}

void dfs(int node)
{
    for (const auto &[u, cost, idx] : g[node])
    {
        dfs(u);
    }

    std::priority_queue <Range> pq;
    for (const auto &[u, cost, idx] : g[node])
    {
        treeEdges.update(in[u], out[u], idx);
        treeNodes.update(in[u], out[u], cost);
        remaining[idx] = cost;

        SegmentTreeNodes::Node res = treeNodes.query(in[u], out[u]);
        pq.push({res.max, in[u], out[u], res.maxIdx});
    }

    if (pq.empty())
    {
        return;
    }

    if (pq.size() == 1)
    {
        while (pq.top().value > t && cost <= k)
        {
            // std::cout << "whiling1: " << pq.top().value << ' ' << t << '\n' << std::flush;
            auto [value, l, r, idx] = pq.top();
            pq.pop();
            makeLessThan(idx, value, t);
            SegmentTreeNodes::Node res = treeNodes.query(l, r);
            pq.push({res.max, l, r, res.maxIdx});
        }
        
        return;
    }
    
    int atLeast = (t / 2) + (t % 2);
    while (cost <= k)
    {
        auto [value, l, r, idx] = pq.top();
        pq.pop();
        
        if (value + pq.top().value <= t || value <= atLeast)
        {
            SegmentTreeNodes::Node res = treeNodes.query(l, r);
            pq.push({res.max, l, r, res.maxIdx});
            break;
        }

        int target = atLeast;
        if (pq.top().value + atLeast < t)
        {
            target = t - pq.top().value;
        }

        makeLessThan(idx, value, target);
        SegmentTreeNodes::Node res = treeNodes.query(l, r);
        pq.push({res.max, l, r, res.maxIdx});
    }

    while (cost <= k)
    {
        auto [value, l, r, idx] = pq.top();
        pq.pop();
        
        if (value + pq.top().value <= t)
        {
            break;
        }
        
        assert(value + pq.top().value == t + 1);
        int target = value - 1;
        makeLessThan(idx, value, target);
        SegmentTreeNodes::Node res = treeNodes.query(l, r);
        pq.push({res.max, l, r, res.maxIdx});
    }
}

bool check(int val)
{
    cost = 0;
    t = val;
    std::fill(removed + 1, removed + n, false);
    std::fill(remaining + 1, remaining + n, 0);
    treeNodes.build();
    treeEdges.build();
    dfs(1);
    return cost <= k;
}

void solve()
{
    // edges[0].dep = INF;
    // tour.push_back(0);
    // initDFS(1, 0);

    // int last = -1;
    // int curr = 1;

    // while (!check(curr))
    // {
    //     last = curr;
    //     if (curr > 1e9)
    //     {
    //         curr = INF;
    //         break;
    //     }

    //     curr *= 2;
    // }

    int l = -1, r = MAXN, mid;
    while (l < r - 1)
    {
        mid = (l + r) / 2;
        if (!check(mid)) l = mid;
        else r = mid;
    }    

    std::cout << r << '\n';
}

std::mt19937 rng(69420);
void input()
{
    std::cin >> n >> k;
    for (int i = 1 ; i < n ; ++i)
    {
        int u, v, c;
        // u = rng() % i + 1;
        // v = i + 1;
        // c = rng() % 10000;
        std::cin >> u >> v >> c;
        g[u].push_back({v, c, i});
        g[v].push_back({u, c, i});
    }
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIOI();
    input();
    solve();

    return 0;
}